/**
 * 
 */
package com.wieseke.cptk.common.action;

import org.eclipse.jface.resource.ImageDescriptor;

import com.wieseke.cptk.app.CPTKApplication;
import com.wieseke.cptk.common.constants.CommonActionConstants;

/**
 * @author Nicolas Wieseke
 * 
 */
public abstract class CophylogenyViewerActionFactory {

	/**
	 * action factory for zoom reset action
	 */
	public static final CophylogenyViewerActionFactory ZOOM_RESET = new CophylogenyViewerActionFactory() {
		public CommonAction create() {

			String text = CommonActionConstants.ZOOM_RESET_TEXT;
			CommonAction action = new ZoomResetAction(text);
			action.setId(CommonActionConstants.ZOOM_RESET_ID);
			action.setImageDescriptor(ImageDescriptor.createFromFile(
					CPTKApplication.class,
					CommonActionConstants.ZOOM_RESET_IMAGEPATH));
			action.setToolTipText(CommonActionConstants.ZOOM_RESET_TOOLTIPTEXT);
			return action;
		}
	};

	/**
	 * action factory for zoom in action
	 */
	public static final CophylogenyViewerActionFactory ZOOM_IN = new CophylogenyViewerActionFactory() {
		public CommonAction create() {

			String text = CommonActionConstants.ZOOM_IN_TEXT;
			CommonAction action = new ZoomInAction(text);
			action.setId(CommonActionConstants.ZOOM_IN_ID);
			action.setImageDescriptor(ImageDescriptor.createFromFile(
					CPTKApplication.class,
					CommonActionConstants.ZOOM_IN_IMAGEPATH));
			action.setToolTipText(CommonActionConstants.ZOOM_IN_TOOLTIPTEXT);
			return action;
		}
	};

	/**
	 * action factory for zoom out action
	 */
	public static final CophylogenyViewerActionFactory ZOOM_OUT = new CophylogenyViewerActionFactory() {
		public CommonAction create() {

			String text = CommonActionConstants.ZOOM_OUT_TEXT;
			CommonAction action = new ZoomOutAction(text);
			action.setId(CommonActionConstants.ZOOM_OUT_ID);
			action.setImageDescriptor(ImageDescriptor.createFromFile(
					CPTKApplication.class,
					CommonActionConstants.ZOOM_OUT_IMAGEPATH));
			action.setToolTipText(CommonActionConstants.ZOOM_OUT_TOOLTIPTEXT);
			return action;
		}
	};

	/**
	 * action factory for adjust child nodes action
	 */
	public static final CophylogenyViewerActionFactory ADJUST_CHILD_NODES = new CophylogenyViewerActionFactory() {
		public CommonAction create() {

			String text = CommonActionConstants.ADJUST_CHILD_NODES_TEXT;
			CommonAction action = new SelectAdjustChildNodesAction(text);
			action.setId(CommonActionConstants.ADJUST_CHILD_NODES_ID);
			action.setImageDescriptor(ImageDescriptor.createFromFile(
					CPTKApplication.class,
					CommonActionConstants.ADJUST_CHILD_NODES_IMAGEPATH));
			action
					.setToolTipText(CommonActionConstants.ADJUST_CHILD_NODES_TOOLTIPTEXT);
			return action;
		}
	};

	/**
	 * action factory for show host labels action
	 */
	public static final CophylogenyViewerActionFactory SHOW_HOST_LABELS = new CophylogenyViewerActionFactory() {
		public CommonAction create() {

			String text = CommonActionConstants.SHOW_HOST_LABELS_TEXT;
			CommonAction action = new SelectShowHostLabelsAction(text);
			action.setId(CommonActionConstants.SHOW_HOST_LABELS_ID);
			action.setImageDescriptor(ImageDescriptor.createFromFile(
					CPTKApplication.class,
					CommonActionConstants.SHOW_HOST_LABELS_IMAGEPATH));
			action
					.setToolTipText(CommonActionConstants.SHOW_HOST_LABELS_TOOLTIPTEXT);
			return action;
		}
	};

	/**
	 * action factory for show parasite labels action
	 */
	public static final CophylogenyViewerActionFactory SHOW_PARASITE_LABELS = new CophylogenyViewerActionFactory() {
		public CommonAction create() {

			String text = CommonActionConstants.SHOW_PARASITE_LABELS_TEXT;
			CommonAction action = new SelectShowParasiteLabelsAction(text);
			action.setId(CommonActionConstants.SHOW_PARASITE_LABELS_ID);
			action.setImageDescriptor(ImageDescriptor.createFromFile(
					CPTKApplication.class,
					CommonActionConstants.SHOW_PARASITE_LABELS_IMAGEPATH));
			action
					.setToolTipText(CommonActionConstants.SHOW_PARASITE_LABELS_TOOLTIPTEXT);
			return action;
		}
	};

	/**
	 * action factory for show rank labels action
	 */
	public static final CophylogenyViewerActionFactory SHOW_RANK_LABELS = new CophylogenyViewerActionFactory() {
		public CommonAction create() {

			String text = CommonActionConstants.SHOW_RANK_LABELS_TEXT;
			CommonAction action = new SelectShowRankLabelsAction(text);
			action.setId(CommonActionConstants.SHOW_RANK_LABELS_ID);
			action.setImageDescriptor(ImageDescriptor.createFromFile(
					CPTKApplication.class,
					CommonActionConstants.SHOW_RANK_LABELS_IMAGEPATH));
			action
					.setToolTipText(CommonActionConstants.SHOW_RANK_LABELS_TOOLTIPTEXT);
			return action;
		}
	};

	/**
	 * action factory for recalculate node positions action
	 */
	public static final CophylogenyViewerActionFactory RECALCULATE_NODE_POSITIONS = new CophylogenyViewerActionFactory() {
		public CommonAction create() {

			String text = CommonActionConstants.RECALCULATE_NODE_POSITIONS_TEXT;
			CommonAction action = new RearrangeRecalculateNodePositionsAction(
					text);
			action.setId(CommonActionConstants.RECALCULATE_NODE_POSITIONS_ID);
			action
					.setImageDescriptor(ImageDescriptor
							.createFromFile(
									CPTKApplication.class,
									CommonActionConstants.RECALCULATE_NODE_POSITIONS_IMAGEPATH));
			action
					.setToolTipText(CommonActionConstants.RECALCULATE_NODE_POSITIONS_TOOLTIPTEXT);
			return action;
		}
	};

	/**
	 * action factory for recalculate node order action
	 */
	public static final CophylogenyViewerActionFactory RECALCULATE_NODE_ORDER = new CophylogenyViewerActionFactory() {
		public CommonAction create() {

			String text = CommonActionConstants.RECALCULATE_NODE_ORDER_TEXT;
			CommonAction action = new RearrangeRecalculateNodeOrderAction(text);
			action.setId(CommonActionConstants.RECALCULATE_NODE_ORDER_ID);
			action.setImageDescriptor(ImageDescriptor.createFromFile(
					CPTKApplication.class,
					CommonActionConstants.RECALCULATE_NODE_ORDER_IMAGEPATH));
			action
					.setToolTipText(CommonActionConstants.RECALCULATE_NODE_ORDER_TOOLTIPTEXT);
			return action;
		}
	};

	/**
	 * action factory for flip horizontally action
	 */
	public static final CophylogenyViewerActionFactory FLIP_HORIZONTAL = new CophylogenyViewerActionFactory() {
		public CommonAction create() {

			String text = CommonActionConstants.FLIP_HORIZONTAL_TEXT;
			CommonAction action = new RearrangeFlipHorizontalAction(text);
			action.setId(CommonActionConstants.FLIP_HORIZONTAL_ID);
			action.setImageDescriptor(ImageDescriptor.createFromFile(
					CPTKApplication.class,
					CommonActionConstants.FLIP_HORIZONTAL_IMAGEPATH));
			action
					.setToolTipText(CommonActionConstants.FLIP_HORIZONTAL_TOOLTIPTEXT);
			return action;
		}
	};

	/**
	 * action factory for flip vertically action
	 */
	public static final CophylogenyViewerActionFactory FLIP_VERTICAL = new CophylogenyViewerActionFactory() {
		public CommonAction create() {

			String text = CommonActionConstants.FLIP_VERTICAL_TEXT;
			CommonAction action = new RearrangeFlipVerticalAction(text);
			action.setId(CommonActionConstants.FLIP_VERTICAL_ID);
			action.setImageDescriptor(ImageDescriptor.createFromFile(
					CPTKApplication.class,
					CommonActionConstants.FLIP_VERTICAL_IMAGEPATH));
			action
					.setToolTipText(CommonActionConstants.FLIP_VERTICAL_TOOLTIPTEXT);
			return action;
		}
	};

	/**
	 * action factory for rotate left action
	 */
	public static final CophylogenyViewerActionFactory ROTATE_LEFT = new CophylogenyViewerActionFactory() {
		public CommonAction create() {

			String text = CommonActionConstants.ROTATE_LEFT_TEXT;
			CommonAction action = new RearrangeRotateLeftAction(text);
			action.setId(CommonActionConstants.ROTATE_LEFT_ID);
			action.setImageDescriptor(ImageDescriptor.createFromFile(
					CPTKApplication.class,
					CommonActionConstants.ROTATE_LEFT_IMAGEPATH));
			action
					.setToolTipText(CommonActionConstants.ROTATE_LEFT_TOOLTIPTEXT);
			return action;
		}
	};

	/**
	 * action factory for rotate right action
	 */
	public static final CophylogenyViewerActionFactory ROTATE_RIGHT = new CophylogenyViewerActionFactory() {
		public CommonAction create() {

			String text = CommonActionConstants.ROTATE_RIGHT_TEXT;
			CommonAction action = new RearrangeRotateRightAction(text);
			action.setId(CommonActionConstants.ROTATE_RIGHT_ID);
			action.setImageDescriptor(ImageDescriptor.createFromFile(
					CPTKApplication.class,
					CommonActionConstants.ROTATE_RIGHT_IMAGEPATH));
			action
					.setToolTipText(CommonActionConstants.ROTATE_RIGHT_TOOLTIPTEXT);
			return action;
		}
	};

	/**
	 * action factory for print action
	 */
	public static final CophylogenyViewerActionFactory PRINT = new CophylogenyViewerActionFactory() {
		public CommonAction create() {

			String text = CommonActionConstants.PRINT_TEXT;
			CommonAction action = new PrintAction(text);
			action.setId(CommonActionConstants.PRINT_ID);
			action.setImageDescriptor(ImageDescriptor.createFromFile(
					CPTKApplication.class,
					CommonActionConstants.PRINT_IMAGEPATH));
			action.setToolTipText(CommonActionConstants.PRINT_TOOLTIPTEXT);
			return action;
		}
	};

	/**
	 * action factory for print action
	 */
	public static final CophylogenyViewerActionFactory EXPORT_SVG = new CophylogenyViewerActionFactory() {
		public CommonAction create() {

			String text = CommonActionConstants.EXPORT_SVG_TEXT;
			CommonAction action = new ExportSvgAction(text);
			action.setId(CommonActionConstants.EXPORT_SVG_ID);
			action.setImageDescriptor(ImageDescriptor.createFromFile(
					CPTKApplication.class,
					CommonActionConstants.EXPORT_SVG_IMAGEPATH));
			action.setToolTipText(CommonActionConstants.EXPORT_SVG_TOOLTIPTEXT);
			return action;
		}
	};

	/**
	 * crates the action
	 * 
	 * @return the action
	 */
	public abstract CommonAction create();
}
