/**
 * 
 */
package com.wieseke.cptk.input.action;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.jface.action.ActionContributionItem;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.action.IMenuCreator;
import org.eclipse.jface.util.IPropertyChangeListener;
import org.eclipse.jface.util.PropertyChangeEvent;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.ui.IWorkbenchPart;

import com.wieseke.cptk.input.constants.InputViewerConstants;

/**
 * @author Nicolas Wieseke
 * 
 */
public class SelectEditModeSelectionAction extends InputPropertyListenerAction
		implements IMenuCreator {

	/**
	 * the menu
	 */
	private Menu menu;
	/**
	 * the actions
	 */
	private List<IAction> actions;

	/**
	 * default constructor
	 * 
	 * @param text
	 */
	public SelectEditModeSelectionAction(String text) {
		super(text);
		setMenuCreator(this);
		this.actions = new ArrayList<IAction>();
		this.propertyChangeListener = new IPropertyChangeListener() {
			/*
			 * (non-Javadoc)
			 * 
			 * @see
			 * org.eclipse.jface.util.IPropertyChangeListener#propertyChange
			 * (org.eclipse.jface.util.PropertyChangeEvent)
			 */
			
			public void propertyChange(PropertyChangeEvent event) {
				if (event.getProperty().equals(
						InputViewerConstants.PROPERTY_EDIT_MODE)
						&& !event.getNewValue().equals(event.getOldValue())) {
					updateImageDescriptor();
				}
			}
		};
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.action.IMenuCreator#dispose()
	 */
	
	public void dispose() {
		if (menu != null) {
			menu.dispose();
			menu = null;
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * org.eclipse.jface.action.IMenuCreator#getMenu(org.eclipse.swt.widgets
	 * .Menu)
	 */
	
	public Menu getMenu(Menu parent) {
		return null;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * org.eclipse.jface.action.IMenuCreator#getMenu(org.eclipse.swt.widgets
	 * .Control)
	 */
	
	public Menu getMenu(Control parent) {
		if (menu != null)
			menu.dispose();

		menu = new Menu(parent);
		for (IAction action : this.actions) {
			ActionContributionItem item = new ActionContributionItem(action);
			item.fill(menu, -1);
		}
		return menu;
	}

	/**
	 * adds an action to the menu
	 * 
	 * @param action
	 * @return true if successful
	 */
	public boolean addActionToMenu(IAction action) {
		return this.actions.add(action);
	}

	/**
	 * updates the image descriptor
	 */
	public void updateImageDescriptor() {
		for (IAction action : this.actions) {
			if (action.isChecked()) {
				this.setImageDescriptor(action.getImageDescriptor());
			}
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.action.Action#run()
	 */
	
	public void run() {
		for (int i = 0; i < this.actions.size(); i++) {
			IAction action = this.actions.get(i);
			if (action.isChecked()) {
				this.actions.get((i + 1) % this.actions.size()).run();
				break;
			}
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * com.wieseke.cptk.input.action.InputPropertyListenerAction#selectionChanged
	 * (org.eclipse.ui.IWorkbenchPart, org.eclipse.jface.viewers.ISelection)
	 */
	
	public void selectionChanged(IWorkbenchPart part, ISelection selection) {
		super.selectionChanged(part, selection);
		updateImageDescriptor();
	}
}