/**
 * 
 */
package com.wieseke.cptk.input.operation;

import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.commands.operations.AbstractOperation;
import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.jface.window.Window;
import org.eclipse.jface.wizard.WizardDialog;

import com.wieseke.cptk.input.dao.InputHostNode;
import com.wieseke.cptk.input.viewer.InputCophylogenyViewer;
import com.wieseke.cptk.input.wizards.ChangeHostRankWizard;

/**
 * @author Nicolas Wieseke
 * 
 */
public class EditChangeHostRankOperation extends AbstractOperation {

	private InputCophylogenyViewer viewer;
	private InputHostNode node;
	private int oldRank;
	private int newRank;

	public EditChangeHostRankOperation(InputCophylogenyViewer viewer, InputHostNode node) {
		super("change rank of node " + node.getLabel());
		this.viewer = viewer;
		this.node = node;
		this.oldRank = node.getRank();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * org.eclipse.core.commands.operations.AbstractOperation#execute(org.eclipse
	 * .core.runtime.IProgressMonitor, org.eclipse.core.runtime.IAdaptable)
	 */
	
	public IStatus execute(IProgressMonitor monitor, IAdaptable info) throws ExecutionException {
		ChangeHostRankWizard wizard = new ChangeHostRankWizard(this.node);
		WizardDialog dialog = new WizardDialog(this.viewer.getShell(), wizard);
		dialog.create();
		dialog.open();
		if (dialog.getReturnCode() == Window.OK && this.oldRank != wizard.getRank()) {
			this.newRank = wizard.getRank();
			return redo(monitor, info);
		}
		return Status.CANCEL_STATUS;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * org.eclipse.core.commands.operations.AbstractOperation#redo(org.eclipse
	 * .core.runtime.IProgressMonitor, org.eclipse.core.runtime.IAdaptable)
	 */
	
	public IStatus redo(IProgressMonitor monitor, IAdaptable info) throws ExecutionException {
		this.node.setRank(this.newRank);
		this.viewer.redraw();
		this.viewer.setDirty(true);
		return Status.OK_STATUS;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * org.eclipse.core.commands.operations.AbstractOperation#undo(org.eclipse
	 * .core.runtime.IProgressMonitor, org.eclipse.core.runtime.IAdaptable)
	 */
	
	public IStatus undo(IProgressMonitor monitor, IAdaptable info) throws ExecutionException {
		this.node.setRank(this.oldRank);
		this.viewer.redraw();
		this.viewer.setDirty(true);
		return Status.OK_STATUS;
	}

}
