/**
 * 
 */
package com.wieseke.cptk.input.operation;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.commands.operations.AbstractOperation;
import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;

import com.wieseke.cptk.input.dao.InputHostNode;
import com.wieseke.cptk.input.dao.InputParasiteNode;
import com.wieseke.cptk.input.viewer.InputCophylogenyViewer;

/**
 * @author Nicolas Wieseke
 * 
 */
public class EditDeleteParasiteAssociationsOperation extends AbstractOperation {

	private InputCophylogenyViewer viewer;
	private InputParasiteNode node;
	private List<InputHostNode> oldAssociations;

	public EditDeleteParasiteAssociationsOperation(
			InputCophylogenyViewer viewer, InputParasiteNode node) {
		super("delete associations of node " + node.getLabel());
		this.viewer = viewer;
		this.node = node;
		this.oldAssociations = new ArrayList<InputHostNode>();
		for (InputHostNode hostNode : node.getAssociations()) {
			this.oldAssociations.add(hostNode);
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * org.eclipse.core.commands.operations.AbstractOperation#execute(org.eclipse
	 * .core.runtime.IProgressMonitor, org.eclipse.core.runtime.IAdaptable)
	 */
	
	public IStatus execute(IProgressMonitor monitor, IAdaptable info)
			throws ExecutionException {
		// if (!this.node.isLeaf())
		// return Status.CANCEL_STATUS;
		return redo(monitor, info);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * org.eclipse.core.commands.operations.AbstractOperation#redo(org.eclipse
	 * .core.runtime.IProgressMonitor, org.eclipse.core.runtime.IAdaptable)
	 */
	
	public IStatus redo(IProgressMonitor monitor, IAdaptable info)
			throws ExecutionException {
		this.node.getAssociations().clear();
		this.viewer.updateAssociations();
		this.viewer.redraw();
		this.viewer.setDirty(true);
		return Status.OK_STATUS;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * org.eclipse.core.commands.operations.AbstractOperation#undo(org.eclipse
	 * .core.runtime.IProgressMonitor, org.eclipse.core.runtime.IAdaptable)
	 */
	
	public IStatus undo(IProgressMonitor monitor, IAdaptable info)
			throws ExecutionException {
		for (InputHostNode hostNode : this.oldAssociations) {
			this.node.getAssociations().add(hostNode);
		}
		this.viewer.updateAssociations();
		this.viewer.redraw();
		this.viewer.setDirty(true);
		return Status.OK_STATUS;
	}

}
