/**
 * 
 */
package com.wieseke.cptk.input.operation;

import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.commands.operations.AbstractOperation;
import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;

import com.wieseke.cptk.input.dao.InputParasiteNode;
import com.wieseke.cptk.input.viewer.InputCophylogenyViewer;
import com.wieseke.cptk.input.viewer.InputTreeArrangement;

/**
 * @author Nicolas Wieseke
 * 
 */
public class EditDeleteParasiteNodeOperation extends AbstractOperation {

	private InputCophylogenyViewer viewer;
	private InputParasiteNode node;
	private int index;
	InputTreeArrangement arrangement;

	public EditDeleteParasiteNodeOperation(InputCophylogenyViewer viewer, InputParasiteNode node) {
		super("delete node " + node.getLabel());
		this.viewer = viewer;
		this.node = node;
		this.index = node.getParent().getChildren().indexOf(node);
		this.arrangement = new InputTreeArrangement(this.viewer.getCophylogeny());
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * org.eclipse.core.commands.operations.AbstractOperation#execute(org.eclipse
	 * .core.runtime.IProgressMonitor, org.eclipse.core.runtime.IAdaptable)
	 */
	
	public IStatus execute(IProgressMonitor monitor, IAdaptable info) throws ExecutionException {
		if (this.node.getParent() == null)
			return Status.CANCEL_STATUS;
		return redo(monitor, info);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * org.eclipse.core.commands.operations.AbstractOperation#redo(org.eclipse
	 * .core.runtime.IProgressMonitor, org.eclipse.core.runtime.IAdaptable)
	 */
	
	public IStatus redo(IProgressMonitor monitor, IAdaptable info) throws ExecutionException {
		InputParasiteNode parent = this.node.getParent();
		parent.getChildren().remove(this.node);
		for (int i = this.node.getChildren().size() - 1; i >= 0; i--) {
			InputParasiteNode child = this.node.getChildren().get(i);
			parent.getChildren().add(this.index, child);
			child.setParent(parent);
		}
		this.viewer.initCophylogeny(true, this.arrangement);
		this.viewer.getCophylogeny().recountAndRenumber();
		this.viewer.setDirty(true);
		return Status.OK_STATUS;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * org.eclipse.core.commands.operations.AbstractOperation#undo(org.eclipse
	 * .core.runtime.IProgressMonitor, org.eclipse.core.runtime.IAdaptable)
	 */
	
	public IStatus undo(IProgressMonitor monitor, IAdaptable info) throws ExecutionException {
		InputParasiteNode parent = this.node.getParent();
		for (InputParasiteNode child : this.node.getChildren()) {
			parent.getChildren().remove(child);
			child.setParent(this.node);
		}
		parent.getChildren().add(this.index, this.node);
		this.viewer.initCophylogeny(true, this.arrangement);
		this.viewer.getCophylogeny().recountAndRenumber();
		this.viewer.setDirty(true);
		return Status.OK_STATUS;
	}

}
