/**
 * 
 */
package com.wieseke.cptk.input.viewer;

import java.awt.Graphics2D;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.MouseListener;
import org.eclipse.swt.events.MouseMoveListener;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.graphics.Point;

import com.wieseke.cptk.common.api.IPositionNode;
import com.wieseke.cptk.common.viewer.CophylogenyViewer;
import com.wieseke.cptk.common.viewer.NodeControl;
import com.wieseke.cptk.input.action.EditAddNodeAction;
import com.wieseke.cptk.input.action.EditChangeLabelAction;
import com.wieseke.cptk.input.action.EditChangeRankAction;
import com.wieseke.cptk.input.action.EditCutSubtreeAction;
import com.wieseke.cptk.input.action.EditDeleteAssociationsAction;
import com.wieseke.cptk.input.action.EditDeleteNodeAction;
import com.wieseke.cptk.input.action.EditRearrangeChildPositionAction;
import com.wieseke.cptk.input.constants.InputViewerConstants;
import com.wieseke.cptk.input.dao.InputNode;
import com.wieseke.cptk.input.selection.InputNodeControlSelection;

/**
 * @author Nicolas Wieseke
 * 
 */
public abstract class InputNodeControl extends NodeControl {

	/**
	 * the color of the sign that indicates if the node is not expanded
	 */
	protected Color foldSignColor;
	/**
	 * the fill color of the node visualization
	 */
	protected Color nodeColor;
	/**
	 * the fill color of the node visualization which has the focus
	 */
	protected Color nodeHighlightColor;
	/**
	 * the outline color of the node visualization
	 */
	protected Color nodeOutlineColor;
	/**
	 * the current mouse x position
	 */
	private int mouseXPos;
	/**
	 * the current mouse y position
	 */
	private int mouseYPos;

	/**
	 * default constructor
	 * 
	 * @param parent
	 * @param style
	 * @param node
	 */
	public InputNodeControl(CophylogenyViewer viewer, IPositionNode node,
			NodeControl parent) {
		super(viewer, node, parent);

		this.mouseXPos = 0;
		this.mouseYPos = 0;

		addMouseMoveListener(new MouseMoveListener() {

			public void mouseMove(MouseEvent e) {
				if (e.stateMask == SWT.BUTTON1)
					movePosition(e.x, e.y, false);
				if (InputNodeControl.this.getViewer().getPhiSource() != null) {
					InputNodeControl.this
							.getViewer()
							.setPhiTargetPosition(
									new Point(
											scale(getNode().getPosX()
													+ InputNodeControl.this.width
													/ 2),
											scale(getNode().getPosY()
													+ InputNodeControl.this.height
													/ 2)));
					// inform the viewer that there exists an association
					// between the
					// current node and the selected phiSource
					if (getNode().isLeaf()
							&& InputNodeControl.this.getViewer().getPhiSource() != null) {
						InputHostParasiteNodeControlAssociation association = null;
						if (InputNodeControl.this instanceof InputHostNodeControl
								&& InputNodeControl.this.getViewer()
										.getPhiSource() instanceof InputParasiteNodeControl) {
							association = new InputHostParasiteNodeControlAssociation(
									(InputHostNodeControl) InputNodeControl.this,
									(InputParasiteNodeControl) InputNodeControl.this
											.getViewer().getPhiSource());
						} else if (InputNodeControl.this instanceof InputParasiteNodeControl
								&& InputNodeControl.this.getViewer()
										.getPhiSource() instanceof InputHostNodeControl) {
							association = new InputHostParasiteNodeControlAssociation(
									(InputHostNodeControl) InputNodeControl.this
											.getViewer().getPhiSource(),
									(InputParasiteNodeControl) InputNodeControl.this);
						}
						if (association != null
								&& InputNodeControl.this.getViewer()
										.getAssociations()
										.contains(association)) {
							InputNodeControl.this.getViewer()
									.setPhiAssociationExists(true);
						}
					}
					InputNodeControl.this.viewer.redraw();
				}
			}

		});

		addMouseListener(new MouseListener() {

			public void mouseDoubleClick(MouseEvent e) {
				if (e.button == 1) {
					switch (InputNodeControl.this.getViewer().getEditMode()) {
					case InputViewerConstants.EDIT_MODE_FOLD_NODE:
						toggleExpandFlag();
						InputNodeControl.this.viewer.redraw();
						break;
					case InputViewerConstants.EDIT_MODE_ADD_NODE:
						addNode();
						break;
					case InputViewerConstants.EDIT_MODE_DELETE_NODE:
						deleteNode();
						break;
					case InputViewerConstants.EDIT_MODE_CUT_SUBTREE:
						cutSubtree();
						break;
					case InputViewerConstants.EDIT_MODE_CHANGE_LABEL:
						changeLabel();
						break;
					case InputViewerConstants.EDIT_MODE_CHANGE_RANK:
						changeRank();
						break;
					case InputViewerConstants.EDIT_MODE_ADD_ASSOCIATION:
						addAssociationBegin();
						break;
					case InputViewerConstants.EDIT_MODE_DELETE_ASSOCIATIONS:
						deleteAssociations();
						break;
					case InputViewerConstants.EDIT_MODE_REARRANGE_CHILD_POSITION:
						rearrangeChildPosition();
						break;
					default:
						break;
					}
				}
			}

			public void mouseUp(MouseEvent e) {
				if (e.button == 1 && e.count == 1) {
					movePosition(e.x, e.y, true);
				}
			}

			public void mouseDown(MouseEvent e) {
				InputNodeControl.this.setFocus();
				InputNodeControl.this.viewer
						.setSelection(new InputNodeControlSelection(
								InputNodeControl.this));
				if (e.button == 1) {
					setMousePosition(e.x, e.y);
					if (InputNodeControl.this.getViewer().getPhiSource() != null) {
						addAssociationEnd();
					}
				}
			}

		});

		init();
	}

	/**
	 * initializes all dependent stuff of the InputNodeControl implementation
	 */
	protected abstract void init();

	/**
	 * move the node to the specified position
	 * 
	 * @param x
	 * @param y
	 * @param refresh
	 */
	private void movePosition(int x, int y, boolean refresh) {
		int difX = x - this.mouseXPos;
		int difY = y - this.mouseYPos;

		GC gc = new GC(this);
		if (refresh || difX != 0 || difY != 0) {
			int posX = scale(this.node.getPosX());
			int posY = scale(this.node.getPosY());

			if (posX + difX + scale(this.width) / 2 < this.viewer
					.getClientArea().width
					+ this.viewer.getOrigin().x
					&& posY + difY + scale(this.height) / 2 < this.viewer
							.getClientArea().height
							+ this.viewer.getOrigin().y) {
				if (checkMovePositionDif(difX, difY) == false)
					return;
				if (!getViewer().isAdjustChildNode()) {
					for (NodeControl childNode : childNodeControls) {
						if (((InputNodeControl) childNode)
								.checkMovePositionDif(difX, difY) == false)
							return;
					}
					for (NodeControl childNode : childNodeControls) {
						((InputNodeControl) childNode).movePositionDif(difX,
								difY, gc, refresh);
					}
				}
				this.setPosition(normalize(posX + difX),
						normalize(posY + difY), refresh);
			}
		}
		if (difX != 0 || difY != 0) {
			this.viewer.setDirty(true);
		}
		if (refresh) {
			this.viewer.resetMinSize();
			this.viewer.redraw();
		}
	}

	/**
	 * move the node by specified difference
	 * 
	 * @param difX
	 * @param difY
	 * @param gc
	 * @param forceDraw
	 */
	public void movePositionDif(int difX, int difY, GC gc, boolean forceDraw) {
		if (forceDraw || difX != 0 || difY != 0) {
			for (NodeControl childNode : childNodeControls) {
				((InputNodeControl) childNode).movePositionDif(difX, difY, gc,
						forceDraw);
			}
			int posX = scale(this.node.getPosX());
			int posY = scale(this.node.getPosY());
			if (this.isVisible()) {
				this.setPosition(normalize(posX + difX),
						normalize(posY + difY), forceDraw);
			} else
				this.setPosition(normalize(posX + difX),
						normalize(posY + difY), false);
		}
	}

	/**
	 * check if the node can be moved by specified difference
	 * 
	 * @param difX
	 * @param difY
	 * @return
	 */
	public boolean checkMovePositionDif(int difX, int difY) {
		if (!getViewer().isAdjustChildNode()) {
			for (NodeControl childNode : childNodeControls) {
				if (((InputNodeControl) childNode).checkMovePositionDif(difX,
						difY) == false)
					return false;
			}
		}
		int posX = scale(this.node.getPosX());
		int posY = scale(this.node.getPosY());
		if (posX + difX < 0 || posY + difY < 0)
			return false;
		return true;
	}

	/**
	 * paint the control
	 * 
	 * @param gc
	 */
	protected void paintControl(GC gc) {
		gc.setAntialias(SWT.ON);
		int posX = this.node.getPosX();
		int posY = this.node.getPosY();
		if (scale(posX) != getLocation().x || scale(posY) != getLocation().y
				|| scale(width) != getSize().x || scale(height) != getSize().y)
			setBounds(scale(posX), scale(posY), scale(width), scale(height));
		gc.setBackground(nodeColor);
		if (this.isFocusControl() || this.mouseOverFlag)
			gc.setBackground(this.nodeHighlightColor);
		gc.fillOval(0, 0, scale(width), scale(height));
		gc.setLineWidth(scale(1));
		gc.setForeground(nodeOutlineColor);
		gc.drawOval(scale(1), scale(1), scale(width - 2), scale(height - 2));
		gc.setForeground(foldSignColor);
		if (!this.foldFlag) {
			gc.setLineWidth(scale(2));
			gc.drawLine(scale(width / 2), scale(4), scale(width / 2),
					scale(height - 4));
			gc.drawLine(scale(4), scale(height / 2), scale(width - 4),
					scale(height / 2));
		}
	}

	/**
	 * paint the control
	 * 
	 * @param gc
	 */
	protected void paintControl(Graphics2D g2d) {
		int posX = this.node.getPosX();
		int posY = this.node.getPosY();
		g2d.setColor(new java.awt.Color(nodeColor.getRed(), nodeColor
				.getGreen(), nodeColor.getBlue()));
		g2d.fillOval(posX, posY, width, height);
		g2d.setColor(new java.awt.Color(nodeOutlineColor.getRed(),
				nodeOutlineColor.getGreen(), nodeOutlineColor.getBlue()));
		g2d.drawOval(posX + 1, posY + 1, width - 2, height - 2);
	}

	/**
	 * @param mousePosX
	 * @param mousePosY
	 */
	public void setMousePosition(int mousePosX, int mousePosY) {
		this.mouseXPos = mousePosX;
		this.mouseYPos = mousePosY;
	}

	public InputNode getNode() {
		return (InputNode) super.getNode();
	}

	/**
	 * @return the viewer as InputCophylogenyViewer
	 */
	protected InputCophylogenyViewer getViewer() {
		return (InputCophylogenyViewer) this.viewer;
	}

	/**
	 * insert a new leaf node as a child of the current node
	 */
	public void addNode() {
		EditAddNodeAction action = new EditAddNodeAction(this.getViewer(), this);
		action.run();
	}

	/**
	 * remove the current node and append all the child nodes to the parent of
	 * this node
	 */
	public void deleteNode() {
		EditDeleteNodeAction action = new EditDeleteNodeAction(
				this.getViewer(), this);
		action.run();
	}

	/**
	 * remove the whole subtree starting at this node
	 */
	public void cutSubtree() {
		EditCutSubtreeAction action = new EditCutSubtreeAction(
				this.getViewer(), this);
		action.run();
	}

	/**
	 * change the label of the node
	 */
	public void changeLabel() {
		EditChangeLabelAction action = new EditChangeLabelAction(this
				.getViewer(), this);
		action.run();
	}

	/**
	 * change the rank of the node
	 */
	public void changeRank() {
		EditChangeRankAction action = new EditChangeRankAction(
				this.getViewer(), this);
		action.run();
	}

	/**
	 * begin the add association operation
	 */
	public void addAssociationBegin() {
		getViewer().setPhiSource(this);
		this.viewer.redraw();
	}

	/**
	 * finish the add association operation
	 */
	public abstract void addAssociationEnd();

	/**
	 * cancel the add association operation
	 */
	protected void cancelAddAssociation() {
		getViewer().setPhiSource(null);
		this.viewer.redraw();
	}

	/**
	 * delete all associations form this node
	 */
	public void deleteAssociations() {
		EditDeleteAssociationsAction action = new EditDeleteAssociationsAction(
				this.getViewer(), this);
		action.run();
	}

	/**
	 * rearrange the order of the child nodes
	 */
	private void rearrangeChildPosition() {
		if (!this.node.isLeaf()) {
			EditRearrangeChildPositionAction action = new EditRearrangeChildPositionAction(
					this.getViewer(), this);
			action.run();
		}
	}

	protected Point getNormalizedSize() {
		return new Point(this.width, this.height);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.wieseke.cptk.common.viewer.NodeControl#cancelOperations()
	 */

	public void cancelOperations() {
		if (InputNodeControl.this.getViewer().getPhiSource() != null) {
			cancelAddAssociation();
		}
	}

	/**
	 * @return the width
	 */
	public int getWidth() {
		return width;
	}

	/**
	 * @param width
	 *            the width to set
	 */
	public void setWidth(int width) {
		if (width <= 0)
			width = 1;
		this.width = width;
	}

	/**
	 * @return the height
	 */
	public int getHeight() {
		return height;
	}

	/**
	 * @param height
	 *            the height to set
	 */
	public void setHeight(int height) {
		this.height = height;
	}

	/**
	 * @return the foldSignColor
	 */
	public Color getFoldSignColor() {
		return foldSignColor;
	}

	/**
	 * @param foldSignColor
	 *            the foldSignColor to set
	 */
	public void setFoldSignColor(Color foldSignColor) {
		this.foldSignColor = foldSignColor;
	}

	/**
	 * @return the nodeColor
	 */
	public Color getNodeColor() {
		return nodeColor;
	}

	/**
	 * @param nodeColor
	 *            the nodeColor to set
	 */
	public void setNodeColor(Color nodeColor) {
		this.nodeColor = nodeColor;
	}

	/**
	 * @return the nodeHighlightColor
	 */
	public Color getNodeHighlightColor() {
		return nodeHighlightColor;
	}

	/**
	 * @param nodeHighlightColor
	 *            the nodeHighlightColor to set
	 */
	public void setNodeHighlightColor(Color nodeHighlightColor) {
		this.nodeHighlightColor = nodeHighlightColor;
	}

	/**
	 * @return the nodeOutlineColor
	 */
	public Color getNodeOutlineColor() {
		return nodeOutlineColor;
	}

	/**
	 * @param nodeOutlineColor
	 *            the nodeOutlineColor to set
	 */
	public void setNodeOutlineColor(Color nodeOutlineColor) {
		this.nodeOutlineColor = nodeOutlineColor;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.swt.widgets.Widget#toString()
	 */

	public String toString() {
		return this.node.getNumber() + ":" + this.node.getLabel();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.swt.widgets.Widget#dispose()
	 */

	public void dispose() {
		super.dispose();
	}

}
