/**
 * 
 */
package com.wieseke.cptk.main;

import java.io.FileInputStream;
import java.io.OutputStreamWriter;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import com.wieseke.cptk.input.dao.InputCophylogeny;
import com.wieseke.cptk.input.format.nexus.NexusDeserializer;
import com.wieseke.cptk.output.dao.OutputCophylogeny;
import com.wieseke.cptk.reconstruction.dao.EventType;
import com.wieseke.cptk.reconstruction.dao.ReconstructionCophylogeny;
import com.wieseke.cptk.reconstruction.dao.ReconstructionSolution;

/**
 * @author Nicolas Wieseke
 * 
 */
public class Main {

	/**
	 * @param args
	 */
	@SuppressWarnings("unused")
	public static void main(String[] args) {
		if (args.length == 0 || args[0].equalsIgnoreCase("-h")
				|| args[0].equalsIgnoreCase("-help")) {
			printHelp();
			return;
		}
		try {
			String filename = args[0];

			NexusDeserializer nd = new NexusDeserializer();
			InputCophylogeny cophylogeny = nd.deserialize(new FileInputStream(
					filename), filename);

			ReconstructionCophylogeny reconstructionCophylogeny = new ReconstructionCophylogeny(
					cophylogeny);

			reconstructionCophylogeny.compute(null);

			List<ReconstructionSolution> currentSolutions = reconstructionCophylogeny
					.getCurrentSolutions();
			Map<EventType, ReconstructionSolution> bestSolutions = reconstructionCophylogeny
					.getBestSolutions();

			Iterator<ReconstructionSolution> iter = bestSolutions.values()
					.iterator();
			OutputCophylogeny outputCophylogeny = new OutputCophylogeny(
					reconstructionCophylogeny);
			ReconstructionHelper rh = new ReconstructionHelper(
					outputCophylogeny);
			OutputStreamWriter osw = new OutputStreamWriter(System.out);
			// rh.computeAssociationListsViroBonn(oswPaReco);
			rh.computeAssociationLists(osw, null);

		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	/**
	 * prints the help information
	 * */
	static public void printHelp() {
		System.out
				.println("\nSyntax:\n"
						+ "java -cp .:./* com.wieseke.cptk.main.Main [filename.nex]"
						+ "\n\nFor the java -cp parameter use ';' as classpath separanor on Windows machines (not ':').");
	}
}
