/**
 * 
 */
package com.wieseke.cptk.main;

import java.io.File;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.util.ArrayList;
import java.util.List;
import java.util.Random;

import org.apache.commons.io.FileUtils;

import com.wieseke.cptk.common.api.INode;
import com.wieseke.cptk.common.constants.OptionsConstants;
import com.wieseke.cptk.common.dao.ReconstructionCosts;
import com.wieseke.cptk.common.exceptions.SerializeException;
import com.wieseke.cptk.common.log.CophylogenyLogger;
import com.wieseke.cptk.common.util.CombinationIterator;
import com.wieseke.cptk.common.util.NodeUtils;
import com.wieseke.cptk.input.dao.InputCophylogeny;
import com.wieseke.cptk.input.dao.InputHostNode;
import com.wieseke.cptk.input.dao.InputParasiteNode;
import com.wieseke.cptk.input.format.nexus.NexusSerializer;
import com.wieseke.cptk.input.format.nexus.TreeSerializer;
import com.wieseke.cptk.reconstruction.constants.StatisticConstants;
import com.wieseke.cptk.reconstruction.dao.EventType;
import com.wieseke.cptk.reconstruction.dao.ReconstructionCophylogeny;
import com.wieseke.cptk.reconstruction.dao.ReconstructionElement;
import com.wieseke.cptk.reconstruction.dao.ReconstructionHostNode;
import com.wieseke.cptk.reconstruction.dao.ReconstructionInfo;
import com.wieseke.cptk.reconstruction.dao.ReconstructionSolution;
import com.wieseke.cptk.simulation.SimulationPositionDifferences;
import com.wieseke.cptk.statistics.CoevolutionSimulator;

/**
 * @author Nicolas Wieseke
 * 
 */
public class Simulation {

	/**
	 * @param args
	 *            modified
	 */
	public static void main(String[] args) {
		if (args.length != 0
				&& (args[0].equalsIgnoreCase("-h") || args[0]
						.equalsIgnoreCase("-help"))) {
			printHelp();
			return;
		}
		try {
			String model = StatisticConstants.MODEL_AGE;
			int cycles = 100;
			int startCycleNumber = 0;
			int leaves = 10;
			double hostChoosingProbability = 0.5d;
			double cospeciationProbability = 0.5d;
			double switchProbability = 0.25d;
			double prefHostSwitchAlpha = 0d;
			double multihostProbability = 0d;
			boolean forceHostSwitch = false;
			double stdCosts[] = { -2, 1, 2, 4 };
			boolean mergedLeaveChoosing = false;
			boolean saveNex = true;
			boolean doReconstruction = false;
			String fileext = "";

			for (int i = 0; i < args.length; i++) {
				if (args[i].startsWith("-m".toLowerCase())) {
					int arg = Integer.parseInt(args[i].substring(2));
					switch (arg) {
					case 0:
						model = StatisticConstants.MODEL_MARKOV;
						break;
					case 1:
						model = StatisticConstants.MODEL_AGE;
						break;
					default:
						model = StatisticConstants.MODEL_AGE;
						break;
					}
				}
				if (args[i].startsWith("-c".toLowerCase())) {
					cycles = Integer.parseInt(args[i].substring(2));
				}
				if (args[i].startsWith("-s".toLowerCase())) {
					startCycleNumber = Integer.parseInt(args[i].substring(2));
				}
				if (args[i].startsWith("-l".toLowerCase())) {
					leaves = Integer.parseInt(args[i].substring(2));
					fileext += args[i].substring(2) + "_";
				}
				if (args[i].startsWith("-pco".toLowerCase())) {
					cospeciationProbability = Double.parseDouble(args[i]
							.substring(4));
					fileext += args[i].substring(4) + "_";
				}
				if (args[i].startsWith("-fhs".toLowerCase())) {
					forceHostSwitch = true;
				}
				if (args[i].startsWith("-dlc".toLowerCase())) {
					int arg = Integer.parseInt(args[i].substring(4));
					switch (arg) {
					case 0:
						mergedLeaveChoosing = false;
						break;
					case 1:
						mergedLeaveChoosing = true;
						break;
					default:
						mergedLeaveChoosing = false;
						break;
					}
				}
				if (args[i].startsWith("-psw".toLowerCase())) {
					switchProbability = Double
							.parseDouble(args[i].substring(4));
					fileext += args[i].substring(4) + "_";
				}
				if (args[i].startsWith("-phc".toLowerCase())) {
					hostChoosingProbability = Double.parseDouble(args[i]
							.substring(4));
					fileext += args[i].substring(4) + "_";
				}
				if (args[i].startsWith("-pmh".toLowerCase())) {
					multihostProbability = Double.parseDouble(args[i]
							.substring(4));
					fileext += args[i].substring(4) + "_";
				}
				if (args[i].startsWith("-pphs".toLowerCase())) {
					prefHostSwitchAlpha = Double.parseDouble(args[i]
							.substring(5));
					fileext += args[i].substring(5) + "_";
				}
				if (args[i].startsWith("-o".toLowerCase())) {
					int arg = Integer.parseInt(args[i].substring(2));
					switch (arg) {
					case 0:
						saveNex = false;
						break;
					case 1:
						saveNex = true;
						break;
					default:
						saveNex = true;
						break;
					}
				}
				if (args[i].startsWith("-r".toLowerCase())) {
					int arg = Integer.parseInt(args[i].substring(2));
					switch (arg) {
					case 0:
						doReconstruction = false;
						break;
					case 1:
						doReconstruction = true;
						break;
					default:
						doReconstruction = false;
						break;
					}
				}

			}

			OutputStreamWriter osw = new OutputStreamWriter(
					FileUtils.openOutputStream(new File(fileext
							+ "simulation.log")));
			osw.write("RandomCycle\tCounter\tQuality\tTotalCost\tTotalEvents\tCoCost\tCoEvents\tCoProb\tCoFreq\tSoCost\tSoEvents\tSoProb\tSoFreq\tDuCost\tDuEvents\tDuProb\tDuFreq\tHsCost\tHsEvents\tHsProb\tHsFreq\tRootMapping\n");

			OutputStreamWriter oswSimulationQuality = new OutputStreamWriter(
					FileUtils.openOutputStream(new File(fileext
							+ "simulationQuality.log")));
			oswSimulationQuality
					.write("RandomCycle\thostLeaves\tparasiteLeaves\tsimCo\tsimSo\tsimDu\tsimHs\torgCo\torgSo\torgDu\torgHs\tdiffCo\tdiffSo\tdiffDu\tdiffHs\tscale\tvarAssocRatio_h\tvarAssocRatio_p\tscale_mod\tvarAssocRatio_h_mod\tvarAssocRatio_p_mod\tnoOfAssociations\n");

			OutputStreamWriter oswOverview = null;
			OutputStreamWriter oswPosditionDifferences = null;
			if (doReconstruction) {
				oswOverview = new OutputStreamWriter(
						FileUtils.openOutputStream(new File(fileext
								+ "simulationOverview.log")));
				oswOverview
						.write("RandomCycle\thostLeaves\tparasiteLeaves\tscale\tvarAssocRatio_h\tvarAssocRatio_p\tsimEvents\tsimQuality\torgEvents\torgQuality\tbestEvents\tsimCoDiff\tsimSoDiff\tsimDuDiff\tsimHsDiff\torgCoDiff\torgSoDiff\torgDuDiff\torgHsDiff\treconstructions\n");

				oswPosditionDifferences = new OutputStreamWriter(
						FileUtils.openOutputStream(new File(fileext
								+ "simulationPositionDifferences.log")));
				oswPosditionDifferences
						.write("RandomCycle\ttotalNodes\ttotalExact\ttotalHigher\ttotalLower\ttotalDistinct\tinternalNodes\tinternalExact\tinternalHigher\tinternalLower\tinternalDistinct\texact\thigher\tlower\tdistinct\tdistribution\n");
			}

			for (int i = startCycleNumber; i < cycles; i++) {

				// do the simulation
				CoevolutionSimulator coevolutionSimulator = new CoevolutionSimulator(
						new Random(), cospeciationProbability,
						switchProbability, hostChoosingProbability,
						mergedLeaveChoosing, model, prefHostSwitchAlpha,
						multihostProbability);

				InputCophylogeny cophylogeny = coevolutionSimulator
						.getSimulationInputCophylogeny("simulation" + i, leaves);
				EventType simulatedEvents = coevolutionSimulator.getEvents();
				int noOfAssociations = coevolutionSimulator
						.getNoOfAssociations();

				// if forceHostSwitch is true and no switches occurred then
				// reject the simulation
				if (forceHostSwitch && simulatedEvents.getHostswitches() == 0) {
					i--;
					continue;
				}

				// do the reconstruction with original mapping positions (what
				// can be reconstructed: 1C=1D+2S-problem and
				// sortings-after-hostswitch-problem )
				ReconstructionCophylogeny orgReconstructionCophylogeny = new ReconstructionCophylogeny(
						cophylogeny);
				osw.write(i + "org\t");
				orgReconstructionCophylogeny.compute(osw);
				osw.flush();

				EventType orgEvents = new EventType(
						orgReconstructionCophylogeny.getCurrentSolutions()
								.get(0).getEvents());

				int simHostLeaves = ((cophylogeny.getHostRoot()
						.getChildrenCount() + 2) / 2);
				int simParasiteLeaves = ((cophylogeny.getParasiteRoot()
						.getChildrenCount() + 2) / 2);

				double scale = (double) simParasiteLeaves
						/ (double) simHostLeaves;
				double scale_norm = calcLogarithm(
						scale,
						((double) simParasiteLeaves + (double) simHostLeaves) - 1);

				double varAssociationRatio_h = 0d;
				double varAssociationRatio_h_norm = 0d;
				double varAssociationRatio_p = 0d;
				double varAssociationRatio_p_norm = 0d;

				double mu_h = ((double) noOfAssociations)
						/ ((double) simHostLeaves);
				double mu_p = ((double) noOfAssociations)
						/ ((double) simParasiteLeaves);

				for (INode node : cophylogeny.getNodeList(cophylogeny
						.getHostRoot())) {
					if (node.isLeaf()) {
						List<InputParasiteNode> nodeAssociatedPNodes = cophylogeny
								.getAssociationPartners((InputHostNode) node,
										cophylogeny.getParasiteRoot());
						// get associations for node
						int leafCounter = 0;
						for (INode assocnode : nodeAssociatedPNodes) {
							if (assocnode.isLeaf()) {
								leafCounter++;
							}
						}
						varAssociationRatio_h += Math.pow((leafCounter - mu_h),
								2d);
					}
				}
				varAssociationRatio_h /= (double) simHostLeaves;

				for (INode node : cophylogeny.getNodeList(cophylogeny
						.getParasiteRoot())) {
					if (node.isLeaf()) {
						// get associations for node
						int leafCounter = 0;
						for (INode assocnode : ((InputParasiteNode) node)
								.getAssociations()) {
							if (assocnode.isLeaf()) {
								leafCounter++;
							}
						}
						varAssociationRatio_p += Math.pow(leafCounter - mu_p,
								2d);
					}
				}
				varAssociationRatio_p /= (double) simParasiteLeaves;

				varAssociationRatio_p_norm = Math.min(1, varAssociationRatio_p
						/ (simParasiteLeaves * mu_p));
				varAssociationRatio_h_norm = Math.min(1, varAssociationRatio_h
						/ (simHostLeaves * mu_h));

				oswSimulationQuality.write(i
						+ "\t"
						+ simHostLeaves
						+ "\t"
						+ simParasiteLeaves
						+ "\t"
						+ simulatedEvents.getCospeciations()
						+ "\t"
						+ simulatedEvents.getSortings()
						+ "\t"
						+ simulatedEvents.getDuplications()
						+ "\t"
						+ simulatedEvents.getHostswitches()
						+ "\t"
						+ orgEvents.getCospeciations()
						+ "\t"
						+ orgEvents.getSortings()
						+ "\t"
						+ orgEvents.getDuplications()
						+ "\t"
						+ orgEvents.getHostswitches()
						+ "\t"
						+ (simulatedEvents.getCospeciations() - orgEvents
								.getCospeciations())
						+ "\t"
						+ (simulatedEvents.getSortings() - orgEvents
								.getSortings())
						+ "\t"
						+ (simulatedEvents.getDuplications() - orgEvents
								.getDuplications())
						+ "\t"
						+ (simulatedEvents.getHostswitches() - orgEvents
								.getHostswitches()) + "\t" + scale + "\t"
						+ varAssociationRatio_h + "\t" + varAssociationRatio_p
						+ "\t" + scale_norm + "\t" + varAssociationRatio_h_norm
						+ "\t" + varAssociationRatio_p_norm + "\t" + "\t"
						+ noOfAssociations + "\n");
				oswSimulationQuality.flush();

				// write the .nex file
				if (saveNex) {
					// write org nex
					try {
						OutputStreamWriter ow = new OutputStreamWriter(
								FileUtils.openOutputStream(new File(
										"simulation" + i + ".nex")));
						NexusSerializer ns = new NexusSerializer();
						// delete internal node associations
						cophylogeny.deleteNoneLeafAssociations(cophylogeny
								.getParasiteRoot());
						ow.write(ns.serialize(cophylogeny, false));
						ow.flush();
						ow.close();
					} catch (SerializeException e) {
						CophylogenyLogger.logError(e);
					} catch (IOException e) {
						CophylogenyLogger.logError(e);
					}

					// write org txt
					try {
						OutputStreamWriter ow = new OutputStreamWriter(
								FileUtils.openOutputStream(new File(
										"simulation" + i + ".txt")));
						TreeSerializer ts = new TreeSerializer();
						// delete internal node associations
						cophylogeny.deleteNoneLeafAssociations(cophylogeny
								.getParasiteRoot());
						ow.write(ts.serialize(cophylogeny));
						ow.flush();
						ow.close();
					} catch (SerializeException e) {
						CophylogenyLogger.logError(e);
					} catch (IOException e) {
						CophylogenyLogger.logError(e);
					}

					// write org reco
					try {
						OutputStreamWriter ow = new OutputStreamWriter(
								FileUtils.openOutputStream(new File(
										"simulation" + i + ".sim.reco")));
						ow.write("EVENTS\n");
						EventType events = cophylogeny
								.getReconstructionEvents();
						ow.write(events.getCospeciations() + " "
								+ events.getSortings() + " "
								+ events.getDuplications() + " "
								+ events.getHostswitches() + "\n");
						ow.write("RECONSTRUCTION\n");

						for (INode iNode : cophylogeny.getNodeList(cophylogeny
								.getParasiteRoot())) {
							InputParasiteNode parasite = (InputParasiteNode) iNode;
							for (InputHostNode host : parasite
									.getReconstructionAssociations()) {
								ow.write(parasite.getLabel() + "\t"
										+ host.getLabel() + "\n");
							}
						}

						ow.flush();
						ow.close();
					} catch (IOException e) {
						CophylogenyLogger.logError(e);
					}

					cophylogeny.pruneHostTree();

					// write pruned nex
					try {
						OutputStreamWriter ow = new OutputStreamWriter(
								FileUtils.openOutputStream(new File(
										"simulation" + i + ".pruned.nex")));
						NexusSerializer ns = new NexusSerializer();
						// delete internal node associations
						cophylogeny.deleteNoneLeafAssociations(cophylogeny
								.getParasiteRoot());
						ow.write(ns.serialize(cophylogeny, false));
						ow.flush();
						ow.close();
					} catch (SerializeException e) {
						CophylogenyLogger.logError(e);
					} catch (IOException e) {
						CophylogenyLogger.logError(e);
					}

					// write pruned txt
					try {
						OutputStreamWriter ow = new OutputStreamWriter(
								FileUtils.openOutputStream(new File(
										"simulation" + i + ".pruned.txt")));
						TreeSerializer ts = new TreeSerializer();
						// delete internal node associations
						cophylogeny.deleteNoneLeafAssociations(cophylogeny
								.getParasiteRoot());
						ow.write(ts.serialize(cophylogeny));
						ow.flush();
						ow.close();
					} catch (SerializeException e) {
						CophylogenyLogger.logError(e);
					} catch (IOException e) {
						CophylogenyLogger.logError(e);
					}

					// write pruned reco
					try {
						OutputStreamWriter ow = new OutputStreamWriter(
								FileUtils.openOutputStream(new File(
										"simulation" + i + ".pruned.sim.reco")));
						ow.write("EVENTS\n");
						EventType events = cophylogeny
								.getReconstructionEvents();
						ow.write(events.getCospeciations() + " "
								+ events.getSortings() + " "
								+ events.getDuplications() + " "
								+ events.getHostswitches() + "\n");
						ow.write("RECONSTRUCTION\n");

						for (INode iNode : cophylogeny.getNodeList(cophylogeny
								.getParasiteRoot())) {
							InputParasiteNode parasite = (InputParasiteNode) iNode;
							for (InputHostNode host : parasite
									.getReconstructionAssociations()) {
								ow.write(parasite.getLabel() + "\t"
										+ host.getLabel() + "\n");
							}
						}

						ow.flush();
						ow.close();
					} catch (IOException e) {
						CophylogenyLogger.logError(e);
					}
				}

				// do the computation with std costs
				if (doReconstruction) {
					// set fixed costs for the best reconstruction
					cophylogeny.getOptions().put(
							OptionsConstants.AUTOMATIC_COSTS_ID,
							OptionsConstants.AUTOMATIC_COSTS_0_ID);

					cophylogeny.setCosts(new ReconstructionCosts(stdCosts));

					// delete internal node associations
					cophylogeny.deleteNoneLeafAssociations(cophylogeny
							.getParasiteRoot());

					// compute best reconstruction again (with fixed costs)
					osw.write(i + "std\t");
					ReconstructionCophylogeny stdReconstructionCophylogeny = new ReconstructionCophylogeny(
							cophylogeny);
					stdReconstructionCophylogeny.compute(osw);

					for (ReconstructionSolution stdSolution : stdReconstructionCophylogeny
							.getCurrentSolutions()) {

						oswOverview.write(i + "\t" + simHostLeaves + "\t"
								+ simParasiteLeaves + "\t" + scale + "\t"
								+ varAssociationRatio_h + "\t"
								+ varAssociationRatio_p + "\t"
								+ simulatedEvents + "\t" + orgEvents);
						oswOverview.flush();

						oswOverview.write("\t" + stdSolution.getEvents());
						EventType simulationDifferences = new EventType(
								stdSolution.getEvents().getCospeciations()
										- simulatedEvents.getCospeciations(),
								stdSolution.getEvents().getSortings()
										- simulatedEvents.getSortings(),
								stdSolution.getEvents().getDuplications()
										- simulatedEvents.getDuplications(),
								stdSolution.getEvents().getHostswitches()
										- simulatedEvents.getHostswitches());
						EventType orgDifferences = new EventType(stdSolution
								.getEvents().getCospeciations()
								- orgEvents.getCospeciations(), stdSolution
								.getEvents().getSortings()
								- orgEvents.getSortings(), stdSolution
								.getEvents().getDuplications()
								- orgEvents.getDuplications(), stdSolution
								.getEvents().getHostswitches()
								- orgEvents.getHostswitches());
						oswOverview.write("\t"
								+ simulationDifferences.getCospeciations()
								+ "\t" + simulationDifferences.getSortings()
								+ "\t"
								+ simulationDifferences.getDuplications()
								+ "\t"
								+ simulationDifferences.getHostswitches());
						oswOverview.write("\t"
								+ orgDifferences.getCospeciations() + "\t"
								+ orgDifferences.getSortings() + "\t"
								+ orgDifferences.getDuplications() + "\t"
								+ orgDifferences.getHostswitches());
						oswOverview.write("\t"
								+ stdReconstructionCophylogeny
										.getCurrentSolutions().size());
						oswOverview.write("\n");
						oswOverview.flush();
					}

					List<Integer> rootMappings = stdReconstructionCophylogeny
							.computeBestRootMappings();
					List<SimulationPositionDifferences> positionDifferencesList = new ArrayList<SimulationPositionDifferences>();
					for (Integer rootMapping : rootMappings) {
						ReconstructionInfo reconstructionInfo = stdReconstructionCophylogeny
								.getParasiteNodes()[0].getReconstructionInfo()[rootMapping];
						positionDifferencesList
								.addAll(computeSimulationPositionDifferences(
										orgReconstructionCophylogeny,
										stdReconstructionCophylogeny,
										reconstructionInfo));
					}

					for (SimulationPositionDifferences differences : positionDifferencesList) {
						oswPosditionDifferences.write(i + "\t" + differences
								+ "\n");
					}
				}

				osw.flush();
				oswSimulationQuality.flush();
				if (doReconstruction) {
					oswOverview.flush();
					oswPosditionDifferences.flush();
				}
			}
			osw.close();
			oswSimulationQuality.close();
			if (doReconstruction) {
				oswOverview.close();
				oswPosditionDifferences.close();
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	@SuppressWarnings("unchecked")
	private static List<SimulationPositionDifferences> computeSimulationPositionDifferences(
			ReconstructionCophylogeny orgReconstructionCophylogeny,
			ReconstructionCophylogeny bestReconstructionCophylogeny,
			ReconstructionInfo reconstructionInfo) {

		// compute local differences
		ReconstructionHostNode orgHost = orgReconstructionCophylogeny
				.getParasiteNodes()[reconstructionInfo.getParasiteIndex()]
				.getAssociations().get(0);
		ReconstructionHostNode reconstructedHost = orgReconstructionCophylogeny
				.getHostNodes()[reconstructionInfo.getHostIndex()];
		SimulationPositionDifferences localPositionDifferences = new SimulationPositionDifferences(
				orgReconstructionCophylogeny.getParasiteNodes().length);
		if (orgHost.getNumber() == reconstructedHost.getNumber()) {
			localPositionDifferences.addExactMapping();
		} else if (NodeUtils.isChild(orgHost, reconstructedHost)) {
			localPositionDifferences.addHigherMapping();
			localPositionDifferences.addMappingDifferences(orgHost.getLevel()
					- reconstructedHost.getLevel());
		} else if (NodeUtils.isChild(reconstructedHost, orgHost)) {
			localPositionDifferences.addLowerMapping();
			localPositionDifferences.addMappingDifferences(orgHost.getLevel()
					- reconstructedHost.getLevel());
		} else {
			localPositionDifferences.addDistinctMapping();
		}

		List<SimulationPositionDifferences> positionDifferences = new ArrayList<SimulationPositionDifferences>();
		for (ReconstructionElement reconstructionElement : reconstructionInfo
				.getReconstructions()) {
			if (reconstructionElement.getChildReconstructionInfos() == null
					|| reconstructionElement.getChildReconstructionInfos().length == 0) {
				positionDifferences
						.add(new SimulationPositionDifferences(
								orgReconstructionCophylogeny.getParasiteNodes().length));
				continue;
			}
			List<SimulationPositionDifferences>[] childPositionDifferences = new ArrayList[reconstructionElement
					.getChildReconstructionInfos().length];
			for (int i = 0; i < reconstructionElement
					.getChildReconstructionInfos().length; i++) {
				ReconstructionInfo childReconstructionInfo = reconstructionElement
						.getChildReconstructionInfos()[i];
				childPositionDifferences[i] = computeSimulationPositionDifferences(
						orgReconstructionCophylogeny,
						bestReconstructionCophylogeny, childReconstructionInfo);
			}
			CombinationIterator<SimulationPositionDifferences> childPositionDifferencesCombinations = new CombinationIterator<SimulationPositionDifferences>(
					childPositionDifferences,
					SimulationPositionDifferences.class);

			// calculate the Simulation Position Differences for each
			// combination
			while (childPositionDifferencesCombinations.hasNext()) {
				SimulationPositionDifferences[] childPositionDifferencesCombination = childPositionDifferencesCombinations
						.next();
				SimulationPositionDifferences differences = new SimulationPositionDifferences(
						orgReconstructionCophylogeny.getParasiteNodes().length);
				differences
						.addSimulationPositionDifferences(localPositionDifferences);
				for (SimulationPositionDifferences childDifference : childPositionDifferencesCombination) {
					differences
							.addSimulationPositionDifferences(childDifference);
				}
				positionDifferences.add(differences);

			}
		}
		return positionDifferences;
	}

	/**
	 * 
	 * @param number
	 * @param basis
	 * @return logarithm of number to basis
	 */
	private static double calcLogarithm(double number, double basis) {
		return Math.log10(number) / Math.log10(basis);
	}

	/**
	 * prints the help information
	 * */
	static public void printHelp() {
		System.out
				.println("\nSyntax:\n"
						+ "java -cp .:./* com.wieseke.cptk.main.Simulation [filename.nex] [options]"
						+ "\n\nOptions:\n"
						+ "-mX     \tSpecify the tree model.\n"
						+ "        \tX=0: Markov model (beta = 0)\n"
						+ "        \tX=1: AGE model [standard]\n"
						+ "-lX     \tSpecify the number of total leaves.\n"
						+ "        \tX: a numeric value specifying the number of leaves in the simulated system [standard=10]\n"
						+ "-cX     \tSpecify the number of random cycles.\n"
						+ "        \tX: a numeric value specifying the number of random cycles [standard=100]\n"
						+ "-pcoX   \tSpecify the probability of the cospeciation event.\n"
						+ "        \tX: a double value specifying the probability of the cospeciation event [standard=0.5]\n"
						+ "-pswX   \tSpecify the probability of the switching event.\n"
						+ "        \tX: a double value specifying the probability of the switch event [standard=0.25].\n"
						+ "-phcX   \tSpecify the probability of the switching event (only if -dlc0 is used).\n"
						+ "         \tX: a double value specifying the probability for a speciation beeing from the host tree [standard=0.5].\n"
						+ "-pmhX   \tSpecify the probability of multi-host parasites occurring instead of sortings.\n"
						+ "        \tX: a double value specifying the probability of multi-host parasite events (failure to speciate) [standard=0]\n"
						+ "-dlcX.   \tSpecify the host/parasite tree generation method"
						+ "        \tX=0: choosing the next speciation beeing from host or parasite with given probability (-pch) [standard]\n"
						+ "        \tX=1: choosing the next speciation beeing from host or parasite based on the resp. tree model probability\n"
						+ "-fhsX   \tSpecify if there should be at least one host switch in both (simulation and reconstruction).\n"
						+ "        \tX=0: all simulations and reconstructions will be used [standard]\n"
						+ "        \tX=1: simulations and reconstructions with no host switches will be rejected\n"
						+ "-pphsX  \tSpecify the alpha for preferentiell host switch probability.\n"
						+ "        \tX: a double value specifying the exponent the preferentiell host switch probability [standard=0]\n"
						+ "-vfcvX  \tSpecify if a fixed cost vector should be used.\n"
						+ "        \tX=0: cost vector is computed automatically [standard]\n"
						+ "        \tX=1: standard cost vector (-2, 1, 2, 4) is used\n"
						+ "        \tX=2: simularion probabilities will be used as cost vector\n"
						+ "-sX     \tSpecify the start cycle number.\n"
						+ "        \tX: a numeric value specifying the start cycle number [standard=0]\n"
						+ "-oX     \tSpecify if the respective .nex files should be created.\n"
						+ "        \tX=0: the .nex files won't be saved\n"
						+ "        \tX=1: the .nex files will be saved [standard]\n"
						+ "-rX     \tSpecify if the reconstruction should be computed.\n"
						+ "        \tX=0: the reconstruction will not be computed [standard]\n"
						+ "        \tX=1: the reconstruction will be computed\n"
						+ "\n\nFor the java -cp parameter use ';' as classpath separanor on Windows machines (not ':').");
	}
}