/**
 * 
 */
package com.wieseke.cptk.output.dao;

import java.util.ArrayList;
import java.util.List;

import com.wieseke.cptk.reconstruction.dao.ReconstructionChildMapping;
import com.wieseke.cptk.reconstruction.dao.ReconstructionElement;

/**
 * @author Nic
 * 
 */
public class ChronologyCheck {

	ChronologyNode head;
	List<ChronologyNode> chronologyHostNodes;

	public ChronologyCheck(OutputHostNode hostRoot,
			OutputParasiteNode parasiteRoot) {
		this.head = new ChronologyNode(null, false);
		this.chronologyHostNodes = new ArrayList<ChronologyNode>();
		head.addChild(createChronologyHostStructure(head, hostRoot));
		createChronologyParasiteStructure(null, parasiteRoot);
	}

	private ChronologyNode createChronologyHostStructure(ChronologyNode parent,
			OutputHostNode node) {
		ChronologyNode chronologyHostNode = new ChronologyNode(parent, true);
		this.chronologyHostNodes.add(chronologyHostNode);
		for (OutputHostNode child : node.getChildren()) {
			chronologyHostNode.addChild(createChronologyHostStructure(
					chronologyHostNode, child));
		}
		return chronologyHostNode;
	}

	private ChronologyNode createChronologyParasiteStructure(
			ChronologyNode parent, OutputParasiteNode node) {
		// crate a new ChronologyNode with the given parent
		ChronologyNode chronologyNode = new ChronologyNode(parent, false);
		if (node.getHostIndex() >= 0) {
			if (node.isCospeciation(true)) {
				// if a cospeciation happened then there is only one
				// ChronologyNode for host and parasite
				chronologyNode = this.chronologyHostNodes.get(node
						.getHostIndex());
			} else {
				// otherwise the newly created ChronologyNode is a child of the
				// associated hosts parent node
				this.chronologyHostNodes.get(node.getHostIndex()).getParent()
						.addChild(chronologyNode);
				// and has the associated host as child
				chronologyNode.addChild(this.chronologyHostNodes.get(node
						.getHostIndex()));
			}
		}
		ReconstructionElement reconstruction = node.getReconstruction();
		int childIndex = 0;
		for (OutputParasiteNode child : node.getChildren()) {
			// recursive method invocation for the parasites child nodes
			ChronologyNode chronologyChildNode = createChronologyParasiteStructure(
					chronologyNode, child);
			if (!(chronologyChildNode.isHostNode
					&& chronologyChildNode.getParent() != null && chronologyChildNode
					.getParent() == chronologyNode)) {
				// if the current ChronologyNode as well as the child's
				// ChronologyNode correspond to a host node (a cospeciation
				// occurred in both cases) and there is already a child
				// relationship between these two nodes then do nothing,
				// otherwise
				// the recursively created child ChronologyNode is also a child
				// of the current parasite node, i.e. inherit the the structure
				// of the parasite tree
				chronologyNode.addChild(chronologyChildNode);
			}
			if (reconstruction != null) {
				ReconstructionChildMapping childMapping = reconstruction
						.getChildMappings()[childIndex];
				if (childMapping.isHostswitch()) {
					// if a host switch occurred
					ChronologyNode chronologyTakeofNode = this.chronologyHostNodes
							.get(childMapping.getChildTakeof().get(
									child.getTakeofIndex()));
					ChronologyNode chronologyLandingNode = this.chronologyHostNodes
							.get(childMapping.getChildLanding().get(
									child.getTakeofIndex()));
					// the current parasite node is a child of the take-off host
					// nodes parent and
					chronologyTakeofNode.getParent().addChild(
							chronologyLandingNode);
					// also a child of the landing host nodes parent and
					chronologyLandingNode.getParent().addChild(
							chronologyTakeofNode);
				}
			}
			childIndex++;
		}
		return chronologyNode;
	}

	public boolean isAcyclic() {
		return head.isAcyclic();
	}
}
