/**
 * 
 */
package com.wieseke.cptk.output.viewer;

import java.awt.BasicStroke;
import java.awt.Graphics2D;
import java.awt.geom.AffineTransform;
import java.util.ArrayList;
import java.util.List;

import org.apache.log4j.Logger;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.jface.preference.PreferenceConverter;
import org.eclipse.jface.util.PropertyChangeEvent;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.MouseListener;
import org.eclipse.swt.events.MouseMoveListener;
import org.eclipse.swt.events.PaintEvent;
import org.eclipse.swt.events.PaintListener;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.FontData;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.RGB;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.graphics.Transform;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.ui.PlatformUI;

import com.wieseke.cptk.common.api.IPositionHostNode;
import com.wieseke.cptk.common.api.IPositionParasiteNode;
import com.wieseke.cptk.common.api.ITreeArrangement;
import com.wieseke.cptk.common.constants.OptionsConstants;
import com.wieseke.cptk.common.selection.CophylogenyViewerSelection;
import com.wieseke.cptk.common.viewer.CophylogenyViewer;
import com.wieseke.cptk.common.viewer.NodeControl;
import com.wieseke.cptk.output.constants.OutputViewerConstants;
import com.wieseke.cptk.output.dao.OutputCophylogeny;
import com.wieseke.cptk.output.preferences.OutputPreferenceConstants;

/**
 * @author Nicolas Wieseke
 * 
 */
public class OutputCophylogenyViewer extends CophylogenyViewer {
	/**
	 * Logger
	 */
	static Logger logger = Logger.getLogger(OutputCophylogenyViewer.class);

	private int treeMode;
	private int editMode;

	private String linetype;

	/**
	 * default constructor
	 * 
	 * @param parent
	 * @param style
	 * @param cophylogeny
	 */
	public OutputCophylogenyViewer(Composite parent,
			OutputCophylogeny cophylogeny, String name) {
		super(parent, cophylogeny, name);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.wieseke.cptk.common.viewer.CophylogenyViewer#init()
	 */
	protected void init() {

		IPreferenceStore store = PlatformUI.getPreferenceStore();
		this.hostLineWidth = store
				.getInt(OutputPreferenceConstants.OUTPUT_VIEWER_NAME_HOST_LINEWIDTH);
		this.parasiteLineWidth = store
				.getInt(OutputPreferenceConstants.OUTPUT_VIEWER_NAME_PARASITE_LINEWIDTH);
		this.hostLineColor = new Color(
				Display.getDefault(),
				PreferenceConverter
						.getColor(
								store,
								OutputPreferenceConstants.OUTPUT_VIEWER_NAME_HOST_LINECOLOR));
		this.parasiteLineColor = new Color(
				Display.getDefault(),
				PreferenceConverter
						.getColor(
								store,
								OutputPreferenceConstants.OUTPUT_VIEWER_NAME_PARASITE_LINECOLOR));
		this.hostFont = new Font(Display.getDefault(),
				PreferenceConverter.getFontData(store,
						OutputPreferenceConstants.OUTPUT_VIEWER_NAME_HOST_FONT));
		this.parasiteFont = new Font(
				Display.getDefault(),
				PreferenceConverter
						.getFontData(
								store,
								OutputPreferenceConstants.OUTPUT_VIEWER_NAME_PARASITE_FONT));
		this.hostFontColor = new Color(
				Display.getDefault(),
				PreferenceConverter
						.getColor(
								store,
								OutputPreferenceConstants.OUTPUT_VIEWER_NAME_HOST_FONTCOLOR));
		this.parasiteFontColor = new Color(
				Display.getDefault(),
				PreferenceConverter
						.getColor(
								store,
								OutputPreferenceConstants.OUTPUT_VIEWER_NAME_PARASITE_FONTCOLOR));
		this.linetype = store
				.getString(OutputPreferenceConstants.OUTPUT_VIEWER_NAME_LINETYPE);

		this.treeMode = OutputViewerConstants.TREE_SELECTION_BOTH;
		this.editMode = OutputViewerConstants.EDIT_MODE_FOLD_NODE;

		this.container.addPaintListener(new PaintListener() {

			public void paintControl(PaintEvent e) {
				drawNodeLines(OutputViewerConstants.TREE_SELECTION_HOST, e.gc);
				drawNodeLines(OutputViewerConstants.TREE_SELECTION_PARASITE,
						e.gc);
				if (OutputCophylogenyViewer.this.showHostLabels) {
					drawLabels(OutputViewerConstants.TREE_SELECTION_HOST, e.gc);
				}
				if (OutputCophylogenyViewer.this.showParasiteLabels) {
					drawLabels(OutputViewerConstants.TREE_SELECTION_PARASITE,
							e.gc);
				}
			}
		});
		this.container.addMouseListener(new MouseListener() {

			public void mouseDoubleClick(MouseEvent e) {
			}

			public void mouseDown(MouseEvent e) {
			}

			public void mouseUp(MouseEvent e) {
			}
		});
		this.container.addMouseMoveListener(new MouseMoveListener() {

			public void mouseMove(MouseEvent e) {
			}
		});

		initCophylogeny(true, null);
		this.setFocus();
		setSelection(new CophylogenyViewerSelection(this));
		redraw();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * com.wieseke.cptk.common.viewer.CophylogenyViewer#updatePreferences(org
	 * .eclipse.jface.util.PropertyChangeEvent)
	 */

	protected void updatePreferences(PropertyChangeEvent event) {
		IPreferenceStore store = PlatformUI.getPreferenceStore();
		if (event.getProperty().equals(
				OutputPreferenceConstants.OUTPUT_VIEWER_NAME_LINETYPE)) {
			this.linetype = (String) event.getNewValue();
			return;
		}

		if (event.getProperty().equals(
				OutputPreferenceConstants.OUTPUT_VIEWER_NAME_HOST_NODEWIDTH)) {
			for (NodeControl node : hostNodesControls) {
				((OutputNodeControl) node).setWidth((Integer) event
						.getNewValue());
			}
			return;
		}
		if (event.getProperty().equals(
				OutputPreferenceConstants.OUTPUT_VIEWER_NAME_HOST_NODEHEIGHT)) {
			for (NodeControl node : hostNodesControls) {
				((OutputNodeControl) node).setHeight((Integer) event
						.getNewValue());
			}
			return;
		}
		if (event.getProperty().equals(
				OutputPreferenceConstants.OUTPUT_VIEWER_NAME_HOST_NODECOLOR)) {
			for (NodeControl node : hostNodesControls) {
				((OutputNodeControl) node)
						.setNodeColor(new Color(
								getDisplay(),
								PreferenceConverter
										.getColor(
												store,
												OutputPreferenceConstants.OUTPUT_VIEWER_NAME_HOST_NODECOLOR)));
			}
			return;
		}
		if (event
				.getProperty()
				.equals(OutputPreferenceConstants.OUTPUT_VIEWER_NAME_HOST_NODEOUTLINECOLOR)) {
			for (NodeControl node : hostNodesControls) {
				((OutputNodeControl) node)
						.setNodeOutlineColor(new Color(
								getDisplay(),
								PreferenceConverter
										.getColor(
												store,
												OutputPreferenceConstants.OUTPUT_VIEWER_NAME_HOST_NODEOUTLINECOLOR)));
			}
			return;
		}
		if (event
				.getProperty()
				.equals(OutputPreferenceConstants.OUTPUT_VIEWER_NAME_HOST_NODEHIGHLIGHTCOLOR)) {
			for (NodeControl node : hostNodesControls) {
				((OutputNodeControl) node)
						.setNodeHighlightColor(new Color(
								getDisplay(),
								PreferenceConverter
										.getColor(
												store,
												OutputPreferenceConstants.OUTPUT_VIEWER_NAME_HOST_NODEHIGHLIGHTCOLOR)));
			}
			return;
		}
		if (event
				.getProperty()
				.equals(OutputPreferenceConstants.OUTPUT_VIEWER_NAME_HOST_NODEFOLDSIGNCOLOR)) {
			for (NodeControl node : hostNodesControls) {
				((OutputNodeControl) node)
						.setFoldSignColor(new Color(
								getDisplay(),
								PreferenceConverter
										.getColor(
												store,
												OutputPreferenceConstants.OUTPUT_VIEWER_NAME_HOST_NODEFOLDSIGNCOLOR)));
			}
			return;
		}
		if (event.getProperty().equals(
				OutputPreferenceConstants.OUTPUT_VIEWER_NAME_HOST_LINEWIDTH)) {
			this.hostLineWidth = (Integer) event.getNewValue();
			return;
		}
		if (event.getProperty().equals(
				OutputPreferenceConstants.OUTPUT_VIEWER_NAME_HOST_LINECOLOR)) {
			this.hostLineColor = new Color(
					getDisplay(),
					PreferenceConverter
							.getColor(
									store,
									OutputPreferenceConstants.OUTPUT_VIEWER_NAME_HOST_LINECOLOR));
			return;
		}
		if (event.getProperty().equals(
				OutputPreferenceConstants.OUTPUT_VIEWER_NAME_HOST_FONT)) {
			this.hostFont = new Font(
					getDisplay(),
					PreferenceConverter
							.getFontData(
									store,
									OutputPreferenceConstants.OUTPUT_VIEWER_NAME_HOST_FONT));
			return;
		}
		if (event.getProperty().equals(
				OutputPreferenceConstants.OUTPUT_VIEWER_NAME_HOST_FONTCOLOR)) {
			this.hostFontColor = new Color(
					getDisplay(),
					PreferenceConverter
							.getColor(
									store,
									OutputPreferenceConstants.OUTPUT_VIEWER_NAME_HOST_FONTCOLOR));
			return;
		}

		if (event
				.getProperty()
				.equals(OutputPreferenceConstants.OUTPUT_VIEWER_NAME_PARASITE_NODEWIDTH)) {
			for (NodeControl node : parasiteNodesControls) {
				((OutputNodeControl) node).setWidth((Integer) event
						.getNewValue());
			}
			return;
		}
		if (event
				.getProperty()
				.equals(OutputPreferenceConstants.OUTPUT_VIEWER_NAME_PARASITE_NODEHEIGHT)) {
			for (NodeControl node : parasiteNodesControls) {
				((OutputNodeControl) node).setHeight((Integer) event
						.getNewValue());
			}
			return;
		}
		if (event
				.getProperty()
				.equals(OutputPreferenceConstants.OUTPUT_VIEWER_NAME_PARASITE_NODECOLOR)) {
			for (NodeControl node : parasiteNodesControls) {
				((OutputNodeControl) node)
						.setNodeColor(new Color(
								getDisplay(),
								PreferenceConverter
										.getColor(
												store,
												OutputPreferenceConstants.OUTPUT_VIEWER_NAME_PARASITE_NODECOLOR)));
			}
			return;
		}
		if (event
				.getProperty()
				.equals(OutputPreferenceConstants.OUTPUT_VIEWER_NAME_PARASITE_NODEOUTLINECOLOR)) {
			for (NodeControl node : parasiteNodesControls) {
				((OutputNodeControl) node)
						.setNodeOutlineColor(new Color(
								getDisplay(),
								PreferenceConverter
										.getColor(
												store,
												OutputPreferenceConstants.OUTPUT_VIEWER_NAME_PARASITE_NODEOUTLINECOLOR)));
			}
			return;
		}
		if (event
				.getProperty()
				.equals(OutputPreferenceConstants.OUTPUT_VIEWER_NAME_PARASITE_NODEHIGHLIGHTCOLOR)) {
			for (NodeControl node : parasiteNodesControls) {
				((OutputNodeControl) node)
						.setNodeHighlightColor(new Color(
								getDisplay(),
								PreferenceConverter
										.getColor(
												store,
												OutputPreferenceConstants.OUTPUT_VIEWER_NAME_PARASITE_NODEHIGHLIGHTCOLOR)));
			}
			return;
		}
		if (event
				.getProperty()
				.equals(OutputPreferenceConstants.OUTPUT_VIEWER_NAME_PARASITE_NODEFOLDSIGNCOLOR)) {
			for (NodeControl node : parasiteNodesControls) {
				((OutputNodeControl) node)
						.setFoldSignColor(new Color(
								getDisplay(),
								PreferenceConverter
										.getColor(
												store,
												OutputPreferenceConstants.OUTPUT_VIEWER_NAME_PARASITE_NODEFOLDSIGNCOLOR)));
			}
			return;
		}
		if (event
				.getProperty()
				.equals(OutputPreferenceConstants.OUTPUT_VIEWER_NAME_PARASITE_LINEWIDTH)) {
			this.parasiteLineWidth = (Integer) event.getNewValue();
			return;
		}
		if (event
				.getProperty()
				.equals(OutputPreferenceConstants.OUTPUT_VIEWER_NAME_PARASITE_LINECOLOR)) {
			this.parasiteLineColor = new Color(
					getDisplay(),
					PreferenceConverter
							.getColor(
									store,
									OutputPreferenceConstants.OUTPUT_VIEWER_NAME_PARASITE_LINECOLOR));
			return;
		}
		if (event.getProperty().equals(
				OutputPreferenceConstants.OUTPUT_VIEWER_NAME_PARASITE_FONT)) {
			this.parasiteFont = new Font(
					getDisplay(),
					PreferenceConverter
							.getFontData(
									store,
									OutputPreferenceConstants.OUTPUT_VIEWER_NAME_PARASITE_FONT));
			return;
		}
		if (event
				.getProperty()
				.equals(OutputPreferenceConstants.OUTPUT_VIEWER_NAME_PARASITE_FONTCOLOR)) {
			this.parasiteFontColor = new Color(
					getDisplay(),
					PreferenceConverter
							.getColor(
									store,
									OutputPreferenceConstants.OUTPUT_VIEWER_NAME_PARASITE_FONTCOLOR));
			return;
		}
	}

	/**
	 * initialize the visual representation objects of the cophylogeny
	 * 
	 * @param initNodes
	 *            true if the Controls representing the nodes should be
	 *            initialized
	 * @param arrangement
	 *            the InputTreeArrangement containing information about the tree
	 *            locations or null if the node positions should not be
	 *            recalculated
	 */

	public void initCophylogeny(boolean initNodes, ITreeArrangement arrangement) {
		this.container.setRedraw(false);

		// recalculate the node positions
		if (arrangement != null) {
			this.getCophylogeny().getHostRoot().renumberNodes(0);
			this.getCophylogeny().getHostRoot().calcNodePositions(0);
			this.getCophylogeny().getParasiteRoot().renumberNodes(0);
			this.getCophylogeny().getParasiteRoot().calcNodePositions(0);
		}
		// initialize the visual controls
		if (initNodes) {
			for (NodeControl canvas : this.hostNodesControls) {
				canvas.setMenu(null);
				canvas.dispose();
			}
			this.hostNodesControls.clear();
			initHostNodes(this.getCophylogeny().getHostRoot(), null);
			for (NodeControl canvas : this.parasiteNodesControls) {
				canvas.setMenu(null);
				canvas.dispose();
			}
			this.parasiteNodesControls.clear();
			initParasiteNodes(this.getCophylogeny().getParasiteRoot(), null);
		}
		if (arrangement != null) {
			if (arrangement.getArrangementFlag() == OutputTreeArrangement.ARRANGEMENT_HORIZONTAL) {
				if (arrangement.getPositionFlag() == OutputTreeArrangement.POSITION_FIRST_ROOT_SECOND_CHILDREN) {
					rotateRight(OutputViewerConstants.TREE_SELECTION_BOTH);
				} else {
					rotateLeft(OutputViewerConstants.TREE_SELECTION_BOTH);
				}
				if (arrangement
						.isTreeFlipped(OutputViewerConstants.TREE_SELECTION_HOST)) {
					flipVertical(OutputViewerConstants.TREE_SELECTION_BOTH);
				}
			} else {
				if (arrangement.getPositionFlag() == OutputTreeArrangement.POSITION_FIRST_CHILDREN_SECOND_ROOT) {
				} else {
					flipVertical(OutputViewerConstants.TREE_SELECTION_BOTH);
				}
				if (!arrangement
						.isTreeFlipped(OutputViewerConstants.TREE_SELECTION_HOST)) {
					flipHorizontal(OutputViewerConstants.TREE_SELECTION_BOTH);
				}
			}
			GC gc = new GC(this.container);
			// move the host tree to its original position
			int difX = arrangement
					.getTreeRootPosition(OutputViewerConstants.TREE_SELECTION_HOST).x
					- this.hostNodesControls.get(0).getNode().getPosX();
			int difY = arrangement
					.getTreeRootPosition(OutputViewerConstants.TREE_SELECTION_HOST).y
					- this.hostNodesControls.get(0).getNode().getPosY();
			((OutputNodeControl) this.hostNodesControls.get(0))
					.movePositionDif(scale(difX), scale(difY), gc, true);
			((OutputNodeControl) this.parasiteNodesControls.get(0))
					.movePositionDif(scale(difX), scale(difY), gc, true);
			// move both trees down and to the right so that each node has a
			// position grater than (0,0)
			Rectangle rect = getScaledTreeSize(OutputViewerConstants.TREE_SELECTION_BOTH);
			((OutputNodeControl) this.hostNodesControls.get(0))
					.movePositionDif(Math.max(0, -rect.x),
							Math.max(0, -rect.y), gc, true);
			((OutputNodeControl) this.parasiteNodesControls.get(0))
					.movePositionDif(Math.max(0, -rect.x),
							Math.max(0, -rect.y), gc, true);

		} else if (!getCophylogeny().isPositionsApplied()) {
			// set scale factor to 1.0
			this.scaleFactor = 1d;
			this.getCophylogeny().getHostRoot().calcNodePositions(0);
			this.getCophylogeny().getParasiteRoot().calcNodePositions(0);
			for (NodeControl node : this.hostNodesControls) {
				node.setScaleFactor(scaleFactor);
			}
			for (NodeControl node : this.parasiteNodesControls) {
				node.setScaleFactor(scaleFactor);
			}
			GC gc = new GC(this.container);
			// flip both trees so that the root is on the top and the leaves are
			// at the bottom
			flipHorizontal(OutputViewerConstants.TREE_SELECTION_BOTH);
			flipVertical(OutputViewerConstants.TREE_SELECTION_BOTH);
			// move the two trees so that the root node of the host tree is at
			// position (x,25) and the most left host leaf is at position (25,y)
			Rectangle rect = getScaledTreeSize(OutputViewerConstants.TREE_SELECTION_BOTH);
			((OutputNodeControl) this.hostNodesControls.get(0))
					.movePositionDif(25 - rect.x, 25 - rect.y, gc, false);
			((OutputNodeControl) this.parasiteNodesControls.get(0))
					.movePositionDif(25 - rect.x, 25 - rect.y, gc, false);

			getCophylogeny().setPositionsApplied(true);
		}
		if (arrangement == null) {
			Rectangle rect = getScaledTreeSize(OutputViewerConstants.TREE_SELECTION_BOTH);
			while (this.scaleFactor > 0.01
					&& (rect.height > 768 || rect.width > 1024)) {
				zoomOut();
				rect = getScaledTreeSize(OutputViewerConstants.TREE_SELECTION_BOTH);
			}
		}
		resetMinSize();
		this.container.setRedraw(true);
	}

	public ITreeArrangement getTreeArrangement() {
		ITreeArrangement arrangement = new OutputTreeArrangement(
				this.getCophylogeny());
		return arrangement;
	}

	/**
	 * initialize the nodes of the host tree
	 * 
	 * @param hostNode
	 * @param parent
	 * @return
	 */
	private OutputHostNodeControl initHostNodes(IPositionHostNode hostNode,
			OutputNodeControl parent) {
		OutputHostNodeControl nodeControl = new OutputHostNodeControl(this,
				hostNode, parent);
		int posX = scale(hostNode.getPosX());
		int posY = scale(hostNode.getPosY());
		int width = scale(nodeControl.width);
		int height = scale(nodeControl.height);
		nodeControl.setScaleFactor(this.scaleFactor);
		nodeControl.setBounds(posX, posY, width, height);
		nodeControl.setParentNodeControl(parent);
		if (getNodeContextMenu() != null)
			nodeControl.setMenu(getNodeContextMenu());
		this.hostNodesControls.add(nodeControl);

		for (IPositionHostNode childNode : hostNode.getChildren()) {
			nodeControl.getChildNodeControls().add(
					initHostNodes(childNode, nodeControl));
		}
		return nodeControl;
	}

	/**
	 * initialize the nodes of the host tree
	 * 
	 * @param hostNode
	 * @param parent
	 * @return
	 */
	private OutputParasiteNodeControl initParasiteNodes(
			IPositionParasiteNode parasiteNode, OutputNodeControl parent) {
		OutputParasiteNodeControl nodeControl = new OutputParasiteNodeControl(
				this, parasiteNode, parent);
		int posX = scale(parasiteNode.getPosX());
		int posY = scale(parasiteNode.getPosY());
		int width = scale(nodeControl.width);
		int height = scale(nodeControl.height);
		nodeControl.setScaleFactor(this.scaleFactor);
		nodeControl.setBounds(posX, posY, width, height);
		nodeControl.setParentNodeControl(parent);
		if (getNodeContextMenu() != null)
			nodeControl.setMenu(getNodeContextMenu());
		this.parasiteNodesControls.add(nodeControl);

		for (IPositionParasiteNode childNode : parasiteNode.getChildren()) {
			nodeControl.getChildNodeControls().add(
					initParasiteNodes(childNode, nodeControl));
		}
		return nodeControl;
	}

	/**
	 * draw the lines of the specified tree
	 * 
	 * @param gc
	 */
	private void drawNodeLines(int tree, GC gc) {
		gc.setAntialias(SWT.ON);
		List<NodeControl> nodes = new ArrayList<NodeControl>();
		int arrangement = OutputTreeArrangement.ARRANGEMENT_HORIZONTAL;
		gc.setLineCap(SWT.CAP_ROUND);
		gc.setLineJoin(SWT.JOIN_ROUND);
		switch (tree) {
		case OutputViewerConstants.TREE_SELECTION_HOST:
			gc.setLineWidth(scale(hostLineWidth));
			gc.setForeground(hostLineColor);
			nodes = this.hostNodesControls;
			arrangement = OutputTreeArrangement.getArrangement(this
					.getCophylogeny().getHostRoot());
			break;
		case OutputViewerConstants.TREE_SELECTION_PARASITE:
			gc.setLineWidth(scale(parasiteLineWidth));
			gc.setForeground(parasiteLineColor);
			nodes = this.parasiteNodesControls;
			arrangement = OutputTreeArrangement.getArrangement(this
					.getCophylogeny().getParasiteRoot());
			break;
		default:
			break;
		}

		if (linetype
				.equals(OutputPreferenceConstants.OUTPUT_VIEWER_LINETYPE_ACUTE_ANGLED)) {
			// diagonal lines
			if (tree == OutputViewerConstants.TREE_SELECTION_HOST) {
				for (NodeControl node : nodes) {
					NodeControl parent = node.getParentNodeControl();
					if (parent != null && node.isVisible()) {
						gc.drawLine(scale(parent.getNode().getPosX()
								+ parent.width / 2), scale(parent.getNode()
								.getPosY() + parent.height / 2), scale(node
								.getNode().getPosX() + node.width / 2),
								scale(node.getNode().getPosY() + node.height
										/ 2));
					}
				}
			} else if (tree == OutputViewerConstants.TREE_SELECTION_PARASITE) {
				for (NodeControl node : nodes) {
					NodeControl parent = node.getParentNodeControl();
					if (parent != null && node.isVisible()) {
						Point sourcePoint = new Point(scale(parent.getNode()
								.getPosX() + parent.width / 2), scale(parent
								.getNode().getPosY() + parent.height / 2));
						for (Point point : ((OutputParasiteNodeControl) node)
								.getNode().getEdgePositions()) {
							Point targetPoint = new Point(scale(point.x
									+ parent.width / 2), scale(point.y
									+ parent.height / 2));
							gc.drawLine(sourcePoint.x, sourcePoint.y,
									targetPoint.x, targetPoint.y);
							sourcePoint = targetPoint;
						}
						Point targetPoint = new Point(scale(node.getNode()
								.getPosX() + node.width / 2), scale(node
								.getNode().getPosY() + node.height / 2));
						gc.drawLine(sourcePoint.x, sourcePoint.y,
								targetPoint.x, targetPoint.y);
					}
				}
			}
		} else {
			// orthogonal lines
			if (arrangement == OutputTreeArrangement.ARRANGEMENT_HORIZONTAL) {
				for (NodeControl node : nodes) {
					NodeControl parent = node.getParentNodeControl();
					if (parent != null && node.isVisible()) {
						gc.drawLine(scale(parent.getNode().getPosX()
								+ parent.width / 2), scale(parent.getNode()
								.getPosY() + parent.height / 2), scale(parent
								.getNode().getPosX() + parent.width / 2),
								scale(node.getNode().getPosY() + node.height
										/ 2));
						gc.drawLine(scale(parent.getNode().getPosX()
								+ parent.width / 2), scale(node.getNode()
								.getPosY() + node.height / 2), scale(node
								.getNode().getPosX() + node.width / 2),
								scale(node.getNode().getPosY() + node.height
										/ 2));
					}
				}
			} else if (arrangement == OutputTreeArrangement.ARRANGEMENT_VERTICAL) {
				for (NodeControl node : nodes) {
					NodeControl parent = node.getParentNodeControl();
					if (parent != null && node.isVisible()) {
						gc.drawLine(scale(parent.getNode().getPosX()
								+ parent.width / 2), scale(parent.getNode()
								.getPosY() + parent.height / 2), scale(node
								.getNode().getPosX() + node.width / 2),
								scale(parent.getNode().getPosY()
										+ parent.height / 2));
						gc.drawLine(scale(node.getNode().getPosX() + node.width
								/ 2), scale(parent.getNode().getPosY()
								+ parent.height / 2), scale(node.getNode()
								.getPosX() + node.width / 2), scale(node
								.getNode().getPosY() + node.height / 2));
					}
				}
			}
		}
	}

	/**
	 * draw the lines of the specified tree
	 * 
	 * @param gc
	 */
	private void drawNodeLines(int tree, Graphics2D g2d) {
		List<NodeControl> nodes = new ArrayList<NodeControl>();
		int arrangement = OutputTreeArrangement.ARRANGEMENT_HORIZONTAL;
		RGB rgb;
		switch (tree) {
		case OutputViewerConstants.TREE_SELECTION_HOST:
			g2d.setStroke(new BasicStroke(hostLineWidth));
			rgb = hostLineColor.getRGB();
			g2d.setColor(new java.awt.Color(rgb.red, rgb.green, rgb.blue));
			nodes = this.hostNodesControls;
			arrangement = OutputTreeArrangement.getArrangement(this
					.getCophylogeny().getHostRoot());
			break;
		case OutputViewerConstants.TREE_SELECTION_PARASITE:
			g2d.setStroke(new BasicStroke(parasiteLineWidth));
			rgb = parasiteLineColor.getRGB();
			g2d.setColor(new java.awt.Color(rgb.red, rgb.green, rgb.blue));
			nodes = this.parasiteNodesControls;
			arrangement = OutputTreeArrangement.getArrangement(this
					.getCophylogeny().getParasiteRoot());
			break;
		default:
			break;
		}

		if (linetype
				.equals(OutputPreferenceConstants.OUTPUT_VIEWER_LINETYPE_ACUTE_ANGLED)) {
			// diagonal lines
			// for (NodeControl node : nodes) {
			// NodeControl parent = node.getParentNodeControl();
			// if (parent != null && node.isVisible()) {
			// g2d.drawLine(parent.getNode().getPosX() + parent.width / 2,
			// parent.getNode().getPosY() + parent.height / 2,
			// node.getNode().getPosX() + node.width / 2, node
			// .getNode().getPosY()
			// + node.height / 2);
			// }
			// }
			if (tree == OutputViewerConstants.TREE_SELECTION_HOST) {
				for (NodeControl node : nodes) {
					NodeControl parent = node.getParentNodeControl();
					if (parent != null && node.isVisible()) {
						g2d.drawLine(parent.getNode().getPosX() + parent.width
								/ 2, parent.getNode().getPosY() + parent.height
								/ 2, node.getNode().getPosX() + node.width / 2,
								node.getNode().getPosY() + node.height / 2);
					}
				}
			} else if (tree == OutputViewerConstants.TREE_SELECTION_PARASITE) {
				for (NodeControl node : nodes) {
					NodeControl parent = node.getParentNodeControl();
					if (parent != null && node.isVisible()) {
						Point sourcePoint = new Point(parent.getNode()
								.getPosX() + parent.width / 2, parent.getNode()
								.getPosY() + parent.height / 2);
						for (Point point : ((OutputParasiteNodeControl) node)
								.getNode().getEdgePositions()) {
							Point targetPoint = new Point(point.x
									+ parent.width / 2, point.y + parent.height
									/ 2);
							g2d.drawLine(sourcePoint.x, sourcePoint.y,
									targetPoint.x, targetPoint.y);
							sourcePoint = targetPoint;
						}
						Point targetPoint = new Point(node.getNode().getPosX()
								+ node.width / 2, node.getNode().getPosY()
								+ node.height / 2);
						g2d.drawLine(sourcePoint.x, sourcePoint.y,
								targetPoint.x, targetPoint.y);
					}
				}
			}
		}

		else {
			// orthogonal lines
			if (arrangement == OutputTreeArrangement.ARRANGEMENT_HORIZONTAL) {
				for (NodeControl node : nodes) {
					NodeControl parent = node.getParentNodeControl();
					if (parent != null) {
						g2d.drawLine(parent.getNode().getPosX() + parent.width
								/ 2, parent.getNode().getPosY() + parent.height
								/ 2, parent.getNode().getPosX() + parent.width
								/ 2, node.getNode().getPosY() + node.height / 2);
						g2d.drawLine(parent.getNode().getPosX() + parent.width
								/ 2,
								node.getNode().getPosY() + node.height / 2,
								node.getNode().getPosX() + node.width / 2, node
										.getNode().getPosY() + node.height / 2);
					}
				}
			} else if (arrangement == OutputTreeArrangement.ARRANGEMENT_VERTICAL) {
				for (NodeControl node : nodes) {
					NodeControl parent = node.getParentNodeControl();
					if (parent != null) {
						g2d.drawLine(parent.getNode().getPosX() + parent.width
								/ 2, parent.getNode().getPosY() + parent.height
								/ 2, node.getNode().getPosX() + node.width / 2,
								parent.getNode().getPosY() + parent.height / 2);
						g2d.drawLine(node.getNode().getPosX() + node.width / 2,
								parent.getNode().getPosY() + parent.height / 2,
								node.getNode().getPosX() + node.width / 2, node
										.getNode().getPosY() + node.height / 2);
					}
				}
			}
		}
	}

	/**
	 * draw the labels for the nodes of the given tree
	 * 
	 * @param tree
	 * @param gc
	 */
	private void drawLabels(int tree, GC gc) {
		gc.setAntialias(SWT.ON);
		List<NodeControl> nodes = new ArrayList<NodeControl>();
		int arrangement = OutputTreeArrangement.ARRANGEMENT_HORIZONTAL;
		switch (tree) {
		case OutputViewerConstants.TREE_SELECTION_HOST:
			gc.setForeground(hostFontColor);
			FontData hfd = hostFont.getFontData()[0];
			Font hf = new Font(getDisplay(), hfd.getName(), Math.max(1,
					scale(hfd.getHeight())), hfd.getStyle());
			gc.setFont(hf);
			nodes = this.hostNodesControls;
			arrangement = OutputTreeArrangement.getArrangement(this
					.getCophylogeny().getHostRoot());
			break;
		case OutputViewerConstants.TREE_SELECTION_PARASITE:
			gc.setForeground(parasiteFontColor);
			FontData pfd = parasiteFont.getFontData()[0];
			Font pf = new Font(getDisplay(), pfd.getName(), Math.max(1,
					scale(pfd.getHeight())), pfd.getStyle());
			gc.setFont(pf);
			nodes = this.parasiteNodesControls;
			arrangement = OutputTreeArrangement.getArrangement(this
					.getCophylogeny().getParasiteRoot());
			break;
		default:
			break;
		}

		if (arrangement == OutputTreeArrangement.ARRANGEMENT_HORIZONTAL) {
			for (NodeControl node : nodes) {
				String label = node.getNode().getLabel();
				if (isShowRankLabels())
					label += " (" + node.getNode().getRankText() + ")";
				// TODO: only leaf labels
				// if (node.isVisible()) {
				if (node.isVisible() && node.getNode().isLeaf()) {
					gc.drawText(label, scale(node.getNode().getPosX()
							+ node.width), scale(node.getNode().getPosY() - 6),
							true);
				}
			}
		} else if (arrangement == OutputTreeArrangement.ARRANGEMENT_VERTICAL) {
			Transform t = new Transform(gc.getDevice());
			t.rotate(90f);
			gc.setTransform(t);
			for (NodeControl node : nodes) {
				String label = node.getNode().getLabel();
				if (isShowRankLabels())
					label += " (" + node.getNode().getRankText() + ")";
				// TODO: only leaf labels
				// if (node.isVisible()) {
				if (node.isVisible() && node.getNode().isLeaf()) {
					gc.drawText(label, scale(node.getNode().getPosY()
							+ node.width),
							-scale(node.getNode().getPosX() + 24), true);
				}
			}
			t.rotate(-90f);
			gc.setTransform(t);
			t.dispose();
		}
	}

	/**
	 * draw the labels for the nodes of the given tree
	 * 
	 * @param tree
	 * @param gc
	 */
	private void drawLabels(int tree, Graphics2D g2d) {
		List<NodeControl> nodes = new ArrayList<NodeControl>();
		int arrangement = OutputTreeArrangement.ARRANGEMENT_HORIZONTAL;
		RGB rgb;
		FontData fontData;
		switch (tree) {
		case OutputViewerConstants.TREE_SELECTION_HOST:
			rgb = hostFontColor.getRGB();
			g2d.setColor(new java.awt.Color(rgb.red, rgb.green, rgb.blue));
			fontData = hostFont.getFontData()[0];
			g2d.setFont(new java.awt.Font(fontData.getName(), fontData
					.getStyle(), fontData.getHeight()));
			nodes = this.hostNodesControls;
			arrangement = OutputTreeArrangement.getArrangement(this
					.getCophylogeny().getHostRoot());
			break;
		case OutputViewerConstants.TREE_SELECTION_PARASITE:
			rgb = parasiteFontColor.getRGB();
			g2d.setColor(new java.awt.Color(rgb.red, rgb.green, rgb.blue));
			fontData = parasiteFont.getFontData()[0];
			g2d.setFont(new java.awt.Font(fontData.getName(), fontData
					.getStyle(), fontData.getHeight()));
			nodes = this.parasiteNodesControls;
			arrangement = OutputTreeArrangement.getArrangement(this
					.getCophylogeny().getParasiteRoot());
			break;
		default:
			break;
		}

		if (arrangement == OutputTreeArrangement.ARRANGEMENT_HORIZONTAL) {
			for (NodeControl node : nodes) {
				if (node.getNode().isLeaf()) {
					String label = node.getNode().getLabel();
					if (this.getCophylogeny().getOptions()
							.get(OptionsConstants.RANK_ID) != null
							&& this.getCophylogeny().getOptions()
									.get(OptionsConstants.RANK_ID)
									.equals(OptionsConstants.RANK_1_ID))
						label += " (" + node.getNode().getRankText() + ")";
					g2d.drawString(label,
							node.getNode().getPosX() + node.width, node
									.getNode().getPosY() + 10);
				}
			}
		} else if (arrangement == OutputTreeArrangement.ARRANGEMENT_VERTICAL) {
			AffineTransform t = new AffineTransform();
			t.rotate(90d * Math.PI / 180);
			g2d.setTransform(t);
			for (NodeControl node : nodes) {
				if (node.getNode().isLeaf()) {
					String label = node.getNode().getLabel();
					if (this.getCophylogeny().getOptions()
							.get(OptionsConstants.RANK_ID) != null
							&& this.getCophylogeny().getOptions()
									.get(OptionsConstants.RANK_ID)
									.equals(OptionsConstants.RANK_1_ID))
						label += " (" + node.getNode().getRankText() + ")";
					g2d.drawString(label,
							node.getNode().getPosY() + node.width, -node
									.getNode().getPosX() - 10);
				}
			}
			t.rotate(-90d * Math.PI / 180);
			g2d.setTransform(t);
		}
	}

	/**
	 * flips the tree horizontal
	 * 
	 * @param tree
	 */
	private void flipHorizontal(int tree) {
		this.container.setRedraw(false);
		Rectangle rect = getNormalizedTreeSize(tree);
		switch (tree) {
		case OutputViewerConstants.TREE_SELECTION_HOST:
			for (NodeControl node : this.hostNodesControls) {
				node.flipHorizontal(rect);
			}
			break;
		case OutputViewerConstants.TREE_SELECTION_PARASITE:
			for (NodeControl node : this.parasiteNodesControls) {
				node.flipHorizontal(rect);
			}
			break;
		case OutputViewerConstants.TREE_SELECTION_BOTH:
			for (NodeControl node : this.hostNodesControls) {
				node.flipHorizontal(rect);
			}
			for (NodeControl node : this.parasiteNodesControls) {
				node.flipHorizontal(rect);
			}
			break;
		}
		this.resetMinSize();
		this.container.setRedraw(true);
	}

	/**
	 * flips the tree vertical
	 * 
	 * @param tree
	 */
	private void flipVertical(int tree) {
		this.container.setRedraw(false);
		Rectangle rect = getNormalizedTreeSize(tree);
		switch (tree) {
		case OutputViewerConstants.TREE_SELECTION_HOST:
			for (NodeControl node : this.hostNodesControls) {
				node.flipVertical(rect);
			}
			break;
		case OutputViewerConstants.TREE_SELECTION_PARASITE:
			for (NodeControl node : this.parasiteNodesControls) {
				node.flipVertical(rect);
			}
			break;
		case OutputViewerConstants.TREE_SELECTION_BOTH:
			for (NodeControl node : this.hostNodesControls) {
				node.flipVertical(rect);
			}
			for (NodeControl node : this.parasiteNodesControls) {
				node.flipVertical(rect);
			}
			break;
		}
		this.resetMinSize();
		this.container.setRedraw(true);
	}

	/**
	 * rotate the tree 90 degree to the left
	 * 
	 * @param tree
	 */
	private void rotateLeft(int tree) {
		this.container.setRedraw(false);
		Rectangle rect = getNormalizedTreeSize(tree);
		switch (tree) {
		case OutputViewerConstants.TREE_SELECTION_HOST:
			for (NodeControl node : this.hostNodesControls) {
				node.rotate(rect);
			}
			flipVertical(tree);
			break;
		case OutputViewerConstants.TREE_SELECTION_PARASITE:
			for (NodeControl node : this.parasiteNodesControls) {
				node.rotate(rect);
			}
			flipVertical(tree);
			break;
		case OutputViewerConstants.TREE_SELECTION_BOTH:
			for (NodeControl node : this.hostNodesControls) {
				node.rotate(rect);
			}
			for (NodeControl node : this.parasiteNodesControls) {
				node.rotate(rect);
			}
			flipVertical(tree);
			break;
		}
		this.resetMinSize();
		this.container.setRedraw(true);
	}

	/**
	 * rotate the tree 90 degree to the right
	 * 
	 * @param tree
	 */
	private void rotateRight(int tree) {
		this.container.setRedraw(false);
		Rectangle rect = getNormalizedTreeSize(tree);
		switch (tree) {
		case OutputViewerConstants.TREE_SELECTION_HOST:
			for (NodeControl node : this.hostNodesControls) {
				node.rotate(rect);
			}
			flipHorizontal(OutputViewerConstants.TREE_SELECTION_HOST);
			break;
		case OutputViewerConstants.TREE_SELECTION_PARASITE:
			for (NodeControl node : this.parasiteNodesControls) {
				node.rotate(rect);
			}
			flipHorizontal(OutputViewerConstants.TREE_SELECTION_PARASITE);
			break;
		case OutputViewerConstants.TREE_SELECTION_BOTH:
			for (NodeControl node : this.hostNodesControls) {
				node.rotate(rect);
			}
			for (NodeControl node : this.parasiteNodesControls) {
				node.rotate(rect);
			}
			flipHorizontal(tree);
			break;
		}
		this.resetMinSize();
		this.container.setRedraw(true);
	}

	/**
	 * @return the treeMode
	 */
	public int getTreeMode() {
		return treeMode;
	}

	/**
	 * @param treeMode
	 *            the treeMode to set
	 */
	public void setTreeMode(int treeMode) {
		int oldValue = this.treeMode;
		this.treeMode = treeMode;
		firePropertyChange(OutputViewerConstants.PROPERTY_TREE_MODE, oldValue,
				treeMode);
	}

	/**
	 * @return the editMode
	 */
	public int getEditMode() {
		return editMode;
	}

	/**
	 * @param editMode
	 *            the editMode to set
	 */
	public void setEditMode(int editMode) {
		int oldValue = this.editMode;
		this.editMode = editMode;
		firePropertyChange(OutputViewerConstants.PROPERTY_EDIT_MODE, oldValue,
				editMode);
	}

	/**
	 * Flips the selected tree horizontally.
	 */
	public void flipHorizontal() {
		flipHorizontal(this.treeMode);
		setDirty(true);
	}

	/**
	 * Flips the selected tree vertically.
	 */
	public void flipVertical() {
		flipVertical(this.treeMode);
		setDirty(true);
	}

	/**
	 * Rotate the selected tree to the left.
	 */
	public void rotateLeft() {
		rotateLeft(this.treeMode);
		setDirty(true);
	}

	/**
	 * Rotate the selected tree to the right.
	 */
	public void rotateRight() {
		rotateRight(this.treeMode);
		setDirty(true);
	}

	/**
	 * @return the cophylogeny
	 */
	public OutputCophylogeny getCophylogeny() {
		return (OutputCophylogeny) cophylogeny;
	}

	/**
	 * @param cophylogeny
	 *            the cophylogeny to set
	 */
	public void setCophylogeny(OutputCophylogeny cophylogeny) {
		this.cophylogeny = cophylogeny;
	}

	public void paintContent(Graphics2D g2d) {
		drawNodeLines(OutputViewerConstants.TREE_SELECTION_HOST, g2d);
		drawNodeLines(OutputViewerConstants.TREE_SELECTION_PARASITE, g2d);
		super.paintContent(g2d);
		drawLabels(OutputViewerConstants.TREE_SELECTION_HOST, g2d);
		drawLabels(OutputViewerConstants.TREE_SELECTION_PARASITE, g2d);
	}

	public void showNextReconstruction() {
		if (getCophylogeny().nextReconstruction(true, false)) {
			refreshNodePositions();
			this.setDirty(true);
		}
	}

	public void showPreviousReconstruction() {
		if (getCophylogeny().previousReconstruction(true)) {
			refreshNodePositions();
			this.setDirty(true);
		}
	}

	public void showChoosenReconstruction(long number) {
		if (getCophylogeny().chooseReconstruction(number)) {
			refreshNodePositions();
			this.setDirty(true);
		}
	}

	private void refreshNodePositions() {
		ITreeArrangement arrangement = this.getTreeArrangement();
		this.initCophylogeny(false, arrangement);
		this.setFocus();
		setSelection(new CophylogenyViewerSelection(this));
	}

}
