/**
 * 
 */
package com.wieseke.cptk.reconstruction.util;

import java.util.ArrayList;
import java.util.List;
import java.util.Set;

import com.wieseke.cptk.common.dao.ReconstructionCosts;
import com.wieseke.cptk.reconstruction.dao.EventType;

import de.jtem.numericalMethods.calculus.function.RealFunctionOfSeveralVariables;

/**
 * @author Nic
 * 
 */
public class RestrictedQualityCalculator implements
		RealFunctionOfSeveralVariables {

	private EventType events;
	private List<EventType> knownEvents;

	public RestrictedQualityCalculator(EventType events,
			Set<EventType> knownEvents) {
		this.events = events;
		this.knownEvents = new ArrayList<EventType>();
		this.knownEvents.addAll(knownEvents);
		this.knownEvents.remove(events);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * de.jtem.numericalMethods.calculus.function.RealFunctionOfSeveralVariables
	 * #eval(double[])
	 */
	public double eval(double[] costs) {
		if (!CostCalculator.isValidCostVector(costs)) {
			return Double.MAX_VALUE;
		}
		// TODO change to generate cost sets with sum = 1
		double sum = costs[0] + costs[1] + costs[2] + costs[3];
		costs[0] = costs[0] / sum;
		costs[1] = costs[1] / sum;
		costs[2] = costs[2] / sum;
		costs[3] = costs[3] / sum;
		double cost = costs[0] * events.getCospeciations() + costs[1]
				* events.getSortings() + costs[2] * events.getDuplications()
				+ costs[3] * events.getHostswitches();
		for (EventType retrictionEvents : knownEvents) {
			double retrictionCosts = costs[0]
					* retrictionEvents.getCospeciations() + costs[1]
					* retrictionEvents.getSortings() + costs[2]
					* retrictionEvents.getDuplications() + costs[3]
					* retrictionEvents.getHostswitches();
			if (retrictionCosts <= cost) {
				return Double.MAX_VALUE;
			}
		}
		return QualityCalculator.computeQuality(new ReconstructionCosts(costs),
				this.events).doubleValue();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * de.jtem.numericalMethods.calculus.function.RealFunctionOfSeveralVariables
	 * #getNumberOfVariables()
	 */
	public int getNumberOfVariables() {
		return 4;
	}

}
