/**
 * 
 */
package com.wieseke.cptk.statistics;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Random;

import com.wieseke.cptk.common.dao.ParasiteRank;
import com.wieseke.cptk.common.util.NodeUtils;
import com.wieseke.cptk.input.dao.InputCophylogeny;
import com.wieseke.cptk.input.dao.InputHostNode;
import com.wieseke.cptk.input.dao.InputNode;
import com.wieseke.cptk.input.dao.InputParasiteNode;
import com.wieseke.cptk.reconstruction.constants.StatisticConstants;
import com.wieseke.cptk.reconstruction.dao.EventType;

/**
 * @author Nicolas Wieseke
 * 
 */
public class CoevolutionSimulatorOld {

	private Random random;
	private String model;
	private List<InputHostNode> hostLeaves;
	private List<InputParasiteNode> parasiteLeaves;
	private double cospeciationProbability;
	private double hostVsParasiteLeafChoosingProbability;
	private boolean mergedLeafChoosing;
	private double sortingVsFtsProbability;
	private double switchingVsDuplicationProbability;
	private double prefHostSwitchAlpha;
	private EventType events;
	private int noOfAssociations;

	/**
	 * @author Nicolas Wieseke / Stephanie Keller-Schmidt
	 * 
	 * @param random
	 * @param cospeciationProbability
	 * @param hostVsParasiteLeafChoosingProbability
	 * @param mergedLeaveChoosing
	 * @param model
	 * @param sortingVsFtsProbability
	 * @param duplicationVsSwitchingProbability
	 * @param prefHostSwitchAlpha
	 */
	public CoevolutionSimulatorOld(Random random, double cospeciationProbability,
			double hostVsParasiteLeafChoosingProbability,
			boolean mergedLeaveChoosing, String model,
			double sortingVsFtsProbability,
			double duplicationVsSwitchingProbability, double prefHostSwitchAlpha) {
		if (random == null) {
			this.random = new Random();
		} else {
			this.random = random;
		}
		this.cospeciationProbability = cospeciationProbability;
		this.hostVsParasiteLeafChoosingProbability = hostVsParasiteLeafChoosingProbability;
		this.mergedLeafChoosing = mergedLeaveChoosing;
		this.sortingVsFtsProbability = sortingVsFtsProbability;
		this.switchingVsDuplicationProbability = duplicationVsSwitchingProbability;
		this.prefHostSwitchAlpha = prefHostSwitchAlpha;
		this.model = model;
		this.events = new EventType(0, 0, 0, 0);
		this.noOfAssociations = 0;
	}

	/**
	 * modified
	 * 
	 * @param filename
	 * @param leafNumber
	 * @return
	 */
	public InputCophylogeny getSimulationInputCophylogeny(String filename,
			int leafNumber) {
		InputCophylogeny cophylogeny = InputCophylogeny
				.createNewInputCophylogeny(filename);
		this.hostLeaves = new ArrayList<InputHostNode>();
		this.parasiteLeaves = new ArrayList<InputParasiteNode>();
		this.noOfAssociations = 0;

		createSimulation(cophylogeny, leafNumber);

		cophylogeny.setReconstructionEvents(this.events);

		cophylogeny.getHostRoot().renumberNodes(0);
		cophylogeny.getHostRoot().renameNodes();
		cophylogeny.getParasiteRoot().renumberNodes(0);
		cophylogeny.getParasiteRoot().renameNodes();

		return cophylogeny;
	}

	/**
	 * author: Nicolas Wieseke / Stephanie Keller-Schmidt
	 * 
	 * @param cophylogeny
	 * @param leafNumber
	 */
	private void createSimulation(InputCophylogeny cophylogeny, int leafNumber) {

		InputNode node = null;
		List<InputNode> leavelist = new ArrayList<InputNode>();
		int time = 1;
		int currHostLeafNumber = 1;
		int currParasiteLeafNumber = 1;

		cophylogeny.getParasiteRoot()
				.addAssociations(cophylogeny.getHostRoot());

		cophylogeny.getParasiteRoot().addReconstructionAssociations(
				cophylogeny.getHostRoot());

		(cophylogeny.getParasiteRoot()).setRank(new ParasiteRank(time, time));
		(cophylogeny.getHostRoot()).setRank(time);

		this.parasiteLeaves.add(cophylogeny.getParasiteRoot());
		this.hostLeaves.add(cophylogeny.getHostRoot());

		if (this.mergedLeafChoosing) {
			leavelist.addAll(this.hostLeaves);
			leavelist.addAll(this.parasiteLeaves);

			double q_n[] = getEqualProbabilityDistribution(leavelist);

			node = (InputNode) getRandomSplittingPointFromProbabilityDistribution(
					q_n, leavelist);

		} else {
			// in hosttree
			if (this.random.nextDouble() <= this.hostVsParasiteLeafChoosingProbability) {
				node = cophylogeny.getHostRoot();
				leavelist.addAll(this.hostLeaves);
			}
			// event in parasitetree
			else {
				node = cophylogeny.getParasiteRoot();
				leavelist.addAll(this.parasiteLeaves);
			}
		}

		recursiveCreateSimulation(node, leavelist, time + 1, leafNumber,
				currHostLeafNumber, currParasiteLeafNumber, cophylogeny);
	}// end createRandomTreeByBetaSplitting

	/**
	 * author: Nicolas Wieseke / Stephanie Keller-Schmidt
	 * 
	 * @param node
	 *            current node
	 * @param leavelist
	 *            list of leaves from current tree
	 * @param tree
	 *            defines if host or parasitenode is choosen
	 * @param maxLeafNumber
	 * @param currHostLeafNumber
	 * @param cophylogeny
	 */
	private void recursiveCreateSimulation(InputNode node,
			List<InputNode> leavelist, int time, int maxLeafNumber,
			int currHostLeafNumber, int currParasiteLeafNumber,
			InputCophylogeny cophylogeny) {

		InputNode lNode = null;
		InputNode rNode = null;
		InputParasiteNode plNode = null;
		InputParasiteNode prNode = null;

		// do HOSTSPECIATION
		if (node instanceof InputHostNode) {
			lNode = new InputHostNode();
			rNode = new InputHostNode();

			((InputHostNode) node).setRank(time);
			((InputHostNode) lNode).setRank(time);
			((InputHostNode) rNode).setRank(time);

			node.addChild(lNode);
			lNode.setParent(node);
			node.addChild(rNode);
			rNode.setParent(node);

			List<InputParasiteNode> nodeAssociatedPNodes = cophylogeny
					.getAssociationPartners((InputHostNode) node,
							cophylogeny.getParasiteRoot());

			// pnodes
			for (InputParasiteNode pNode : nodeAssociatedPNodes) {
				if (pNode.isLeaf()) {
					// do COSPECIATION (pspec) with pc=coProbability and if
					// hnode has
					if (this.random.nextDouble() <= this.cospeciationProbability) {
						pNode.setRank(new ParasiteRank(time, time));

						plNode = new InputParasiteNode();
						prNode = new InputParasiteNode();

						plNode.setRank(new ParasiteRank(time, time));
						prNode.setRank(new ParasiteRank(time, time));

						pNode.addChild(plNode);
						plNode.setParent(pNode);
						pNode.addChild(prNode);
						prNode.setParent(pNode);

						plNode.addAssociations((InputHostNode) lNode);
						prNode.addAssociations((InputHostNode) rNode);

						plNode.addReconstructionAssociations((InputHostNode) lNode);
						prNode.addReconstructionAssociations((InputHostNode) rNode);

						// in case of multi-host parasites there occur
						// extinctions on each other associated
						// host which is not the current node
						int extinctions = 0;
						for (InputHostNode hNode : ((InputParasiteNode) pNode)
								.getAssociations()) {
							if (hNode.isLeaf()) {
								if (this.random.nextDouble() <= 0.5) {
									plNode.addAssociations((InputHostNode) hNode);
									plNode.addReconstructionAssociations((InputHostNode) hNode);
								} else {
									prNode.addAssociations((InputHostNode) hNode);
									prNode.addReconstructionAssociations((InputHostNode) hNode);
								}
								extinctions++;

							}
						}

						this.parasiteLeaves.remove(pNode);
						this.parasiteLeaves.add((InputParasiteNode) plNode);
						this.parasiteLeaves.add((InputParasiteNode) prNode);

						currParasiteLeafNumber++;
						this.events = this.events
								.add(new EventType(1, 0, 0, 0));
					}

					// DO SORTING/MULTIHOST ps=1-pc
					else {
						// htree und ptree blätter auf gleiche Länge bringen
						List<InputHostNode> associations = new ArrayList<InputHostNode>();
						List<InputHostNode> mhAssociations = new ArrayList<InputHostNode>();

						if (this.random.nextDouble() < this.sortingVsFtsProbability) {
							associations.add((InputHostNode) lNode);
							pNode.addReconstructionAssociations((InputHostNode) lNode);
							associations.add((InputHostNode) rNode);
							pNode.addReconstructionAssociations((InputHostNode) rNode);
						} else {
							if (this.random.nextDouble() <= 0.5) {
								associations.add((InputHostNode) lNode);
								pNode.addReconstructionAssociations((InputHostNode) lNode);
							} else {
								associations.add((InputHostNode) rNode);
								pNode.addReconstructionAssociations((InputHostNode) rNode);
							}
						}
						// also compute the other associated host nodes in case
						// of a multi-host parasite for re-adding them
						for (InputHostNode hNode : pNode.getAssociations()) {
							if (hNode.isLeaf()) {
								mhAssociations.add(hNode);
							}
						}
						pNode.setAssociations(associations);
						pNode.getAssociations().addAll(mhAssociations);

						// single sortings of the root parasite are not counted
						// unless there are other multi-host associations
						if (pNode.getParent() == null
								&& associations.size() == 1
								&& mhAssociations.size() == 0) {
							// discard all previous associations from the
							// reconstruction associations
							pNode.setReconstructionAssociations(associations);
						} else {
							this.events = this.events.add(new EventType(0,
									associations.size(), 0, 0));
						}
					}
				}
			}

			this.hostLeaves.remove(node);
			this.hostLeaves.add((InputHostNode) lNode);
			this.hostLeaves.add((InputHostNode) rNode);

			currHostLeafNumber++;
		}// end hostspeciation

		// do PARASITESPECIATION
		if (node instanceof InputParasiteNode) {
			// do SWITCH
			if (this.random.nextDouble() <= this.switchingVsDuplicationProbability) {

				lNode = new InputParasiteNode();
				rNode = new InputParasiteNode();

				((InputParasiteNode) node)
						.setRank(new ParasiteRank(time, time));
				((InputParasiteNode) lNode)
						.setRank(new ParasiteRank(time, time));
				((InputParasiteNode) rNode)
						.setRank(new ParasiteRank(time, time));

				node.addChild(lNode);
				lNode.setParent(node);
				node.addChild(rNode);
				rNode.setParent(node);

				// select one of the associated host nodes as take-off site
				// for the switch
				double q_n[] = getEqualProbabilityDistribution(((InputParasiteNode) node)
						.getAssociations());
				InputHostNode hNode1 = (InputHostNode) getRandomSplittingPointFromProbabilityDistribution(
						q_n, ((InputParasiteNode) node).getAssociations());
				// TODO just a check, can be removed later on
				if (!hNode1.isLeaf()) {
					throw new RuntimeException("hNode1 is no leaf");
				}

				// create an ArrayList which contains the prob.distr. q_n
				List<InputHostNode> hostSwitchCandidates = new ArrayList<InputHostNode>();
				hostSwitchCandidates.addAll(this.hostLeaves);
				// remove all already associated hosts
				hostSwitchCandidates.removeAll(((InputParasiteNode) node)
						.getAssociations());
				// if there are no available switching candidates we do a
				// duplication on hNode1 instead

				// do preferential host switching
				if (hostSwitchCandidates.size() == 0) {
					hostSwitchCandidates.add(hNode1);
					q_n = new double[1];
					q_n[0] = 1;
				} else {
					q_n = getPrefAttachProbabilityDistribution(
							hostSwitchCandidates, hNode1);
				}
				// chose hostleave for pnode to switch corresp. to agemodel
				// pdistribution
				InputHostNode hNode2 = (InputHostNode) getRandomSplittingPointFromProbabilityDistribution(
						q_n, hostSwitchCandidates);

				if (this.random.nextDouble() <= 0.5) {
					// lNode ->bekommt assoziation mit hNode1
					((InputParasiteNode) lNode).addAssociations(hNode1);
					// rNode ->bekommt assoziation mit hNode2
					((InputParasiteNode) rNode).addAssociations(hNode2);

					((InputParasiteNode) lNode)
							.addReconstructionAssociations(hNode1);
					((InputParasiteNode) rNode)
							.addReconstructionAssociations(hNode2);

				} else {
					// lNode ->bekommt assoziation mit hNode2
					((InputParasiteNode) lNode).addAssociations(hNode2);
					// rNode ->bekommt assoziation mit hNode1
					((InputParasiteNode) rNode).addAssociations(hNode1);

					((InputParasiteNode) lNode)
							.addReconstructionAssociations(hNode2);
					((InputParasiteNode) rNode)
							.addReconstructionAssociations(hNode1);
				}

				// in case of multi-host parasites there occur
				// extinctions on each other associated host which
				// is not the take-off host hNode1
				int duplications = 0;
				int extinctions = 0;
				for (InputHostNode hNode : ((InputParasiteNode) node)
						.getAssociations()) {
					if (hNode.isLeaf() && !hNode.equals(hNode1)) {
						if (this.random.nextDouble() <= 0.5) {
							((InputParasiteNode) lNode).addAssociations(hNode);
							((InputParasiteNode) lNode)
									.addReconstructionAssociations(hNode);
						} else {
							((InputParasiteNode) rNode).addAssociations(hNode);
							((InputParasiteNode) rNode)
									.addReconstructionAssociations(hNode);
						}
						extinctions++;

					}
				}
				if (hostSwitchCandidates.size() == 1
						&& hostSwitchCandidates.contains(hNode1)) {
					this.events = this.events.add(new EventType(0, 0, 1, 0));
				} else {
					this.events = this.events.add(new EventType(0, 0, 0, 1));
				}
			}
			// do DUPLICATION
			else {

				lNode = new InputParasiteNode();
				rNode = new InputParasiteNode();

				((InputParasiteNode) node)
						.setRank(new ParasiteRank(time, time));
				((InputParasiteNode) lNode)
						.setRank(new ParasiteRank(time, time));
				((InputParasiteNode) rNode)
						.setRank(new ParasiteRank(time, time));

				// select one of the associated host nodes as associated host of
				// the duplication
				double q_n[] = getEqualProbabilityDistribution(((InputParasiteNode) node)
						.getAssociations());
				InputHostNode hNode1 = (InputHostNode) getRandomSplittingPointFromProbabilityDistribution(
						q_n, ((InputParasiteNode) node).getAssociations());
				// TODO just a check, can be removed later on
				if (!hNode1.isLeaf()) {
					throw new RuntimeException("hNode1 is no leaf");
				}

				((InputParasiteNode) lNode).addAssociations(hNode1);
				((InputParasiteNode) rNode).addAssociations(hNode1);

				((InputParasiteNode) lNode)
						.addReconstructionAssociations(hNode1);
				((InputParasiteNode) rNode)
						.addReconstructionAssociations(hNode1);

				node.addChild(lNode);
				lNode.setParent(node);
				node.addChild(rNode);
				rNode.setParent(node);

				// in case of multi-host parasites there occur
				// extinctions on each other associated host which
				// is not the take-off host hNode1
				int duplications = 0;
				int extinctions = 0;
				for (InputHostNode hNode : ((InputParasiteNode) node)
						.getAssociations()) {
					if (hNode.isLeaf() && !hNode.equals(hNode1)) {
						if (this.random.nextDouble() <= 0.5) {
							((InputParasiteNode) lNode).addAssociations(hNode);
							((InputParasiteNode) lNode)
									.addReconstructionAssociations(hNode);
						} else {
							((InputParasiteNode) rNode).addAssociations(hNode);
							((InputParasiteNode) rNode)
									.addReconstructionAssociations(hNode);
						}
						extinctions++;

					}
				}

				this.events = this.events.add(new EventType(0, 0, 1, 0));

			}

			this.parasiteLeaves.remove(node);
			this.parasiteLeaves.add((InputParasiteNode) lNode);
			this.parasiteLeaves.add((InputParasiteNode) rNode);

			currParasiteLeafNumber++;

		}// end parasitespeciation

		// preprocessing for next speciation step
		if (currHostLeafNumber + currParasiteLeafNumber < maxLeafNumber) {

			InputNode node4split;
			time++;

			// create respective leavelist
			leavelist = new ArrayList<InputNode>();
			if (this.mergedLeafChoosing) {
				leavelist.addAll(this.hostLeaves);
				leavelist.addAll(this.parasiteLeaves);
			}// end if mergedleavelist
			else {
				if (this.random.nextDouble() <= this.hostVsParasiteLeafChoosingProbability) {
					leavelist.addAll(this.hostLeaves);
				}
				// event in parasitetree
				else {
					leavelist.addAll(this.parasiteLeaves);
				}
			}// end if nonmergedLeavelist

			// choose next speciation node within the respective model
			double q_n[];
			if (this.model == StatisticConstants.MODEL_AGE) {
				q_n = getProbabilityDistributionForAgeDependentSplitting(
						leavelist, time);
			}// end age modes
			else {
				q_n = getEqualProbabilityDistribution(leavelist);

			}// end yule model
			node4split = (InputNode) getRandomSplittingPointFromProbabilityDistribution(
					q_n, leavelist);

			recursiveCreateSimulation(node4split, leavelist, time,
					maxLeafNumber, currHostLeafNumber, currParasiteLeafNumber,
					cophylogeny);

		} else {
			for (InputHostNode hostLeaf : this.hostLeaves) {
				hostLeaf.setRank(time);
			}
			int noOfAssociations = 0;
			for (InputParasiteNode parasiteLeaf : this.parasiteLeaves) {
				parasiteLeaf.setRank(new ParasiteRank(time, time));
				noOfAssociations += parasiteLeaf.getAssociations().size();
			}
			this.noOfAssociations = noOfAssociations;

		}

	}// end recurseCreateByBetaSplitting

	/**
	 * author: Stephanie Keller-Schmidt
	 * 
	 * getProbabilityDistributionForBetaSplitting(int n) including age
	 * 
	 * @param leavelist
	 * @param time
	 * @return
	 */
	private double[] getProbabilityDistributionForAgeDependentSplitting(
			List<InputNode> leavelist, int time) {

		double q_n[] = new double[leavelist.size()];

		double sum = 0d;
		Iterator<InputNode> itIn = leavelist.iterator();

		while (itIn.hasNext()) {

			InputNode node = itIn.next();
			// which kind of node??? -> typeOf
			if (node instanceof InputHostNode) {
				sum += ((1d / (double) (time - ((InputHostNode) node).getRank())));
			} else {
				sum += (1d / (double) (time - ((InputParasiteNode) node)
						.getRank().getRankFrom()));
			}
		}

		int i = 0;
		Iterator<InputNode> it = leavelist.iterator();

		InputNode n = null;
		while (it.hasNext()) {
			n = it.next();

			if (n instanceof InputHostNode) {
				if (i == 0) {
					q_n[i] = getAgeDependentProbHost(time, (InputHostNode) n,
							sum);
				} else {
					q_n[i] = (q_n[i - 1] + getAgeDependentProbHost(time,
							(InputHostNode) n, sum));
				}
			} else {
				if (i == 0) {
					q_n[i] = getAgeDependentProbParasite(time,
							(InputParasiteNode) n, sum);
				} else {
					q_n[i] = (q_n[i - 1] + getAgeDependentProbParasite(time,
							(InputParasiteNode) n, sum));
				}
			}
			i++;

		}

		return q_n;
	}// end getProbabilityDistributionForAgeDependentSplitting

	/**
	 * author: Stephanie Keller-Schmidt
	 * 
	 * getProbabilityDistributionForBetaSplitting(int n) including age
	 * 
	 * @param leavelist
	 * @return
	 */
	private double[] getEqualProbabilityDistribution(
			List<? extends InputNode> leavelist) {

		double q_n[] = new double[leavelist.size()];

		for (int i = 0; i < leavelist.size(); i++) {
			if (i == 0) {
				q_n[i] = (1d / (double) leavelist.size());
			} else {
				q_n[i] = q_n[i - 1] + (1d / (double) leavelist.size());
			}
		}

		return q_n;
	}// end getEqualProbabilityDistribution

	/**
	 * author: Stephanie Keller-Schmidt
	 * 
	 * getPrefAttachDistributionForBetaSplitting(int n) according to distance of
	 * two nodes -> NodeUtils.java getPathLength(INode node1, INode node2)
	 * 
	 * @param leavelist
	 * @return
	 */
	private double[] getPrefAttachProbabilityDistribution(
			List<? extends InputNode> leavelist, InputHostNode hNode1) {

		double q_n[] = new double[leavelist.size()];

		for (int i = 0; i < leavelist.size(); i++) {
			InputHostNode hNode2 = (InputHostNode) leavelist.get(i);
			if (i == 0) {
				q_n[i] = Math.pow(NodeUtils.getPathLength(hNode1, hNode2),
						prefHostSwitchAlpha * (-1d));
			} else {
				q_n[i] = q_n[i - 1]
						+ (Math.pow(NodeUtils.getPathLength(hNode1, hNode2),
								prefHostSwitchAlpha * (-1d)));
			}
		}

		return q_n;
	}// end getEqualProbabilityDistribution

	/**
	 * author: Stephanie Keller-Schmidt
	 * 
	 * @param q_n
	 * @param leavelist
	 * @return
	 */
	private InputNode getRandomSplittingPointFromProbabilityDistribution(
			double[] q_n, List<? extends InputNode> leavelist) {

		// TODO check this
		double x = this.random.nextDouble() * q_n[q_n.length - 1];
		int i = 0;

		while (i < q_n.length - 1 && q_n[i] < x) {
			i++;
		}
		return leavelist.get(i);
	}// end getRandomSplittingPointFromProbabilityDistribution

	/**
	 * 
	 * @param time
	 * @param n
	 * @param norm
	 */
	private double getAgeDependentProbHost(int time, InputHostNode n,
			double norm) {
		return ((1d / (double) (time - (n.getRank()))) / norm);
	}// end getAgeDependentProbHost

	/**
	 * 
	 * @param time
	 * @param n
	 * @param norm
	 */
	private double getAgeDependentProbParasite(int time, InputParasiteNode n,
			double norm) {
		return (((1d / (double) (time - n.getRank().getRankFrom()))) / norm);
	}// end getAgeDependentProbParasite

	public EventType getEvents() {
		return this.events;
	}

	public int getNoOfAssociations() {
		return this.noOfAssociations;
	}

	public double[] getProbabilities() {
		double sum = this.events.getCospeciations() + this.events.getSortings()
				+ this.events.getDuplications() + this.events.getHostswitches();
		double result[] = { (double) this.events.getCospeciations() / sum,
				(double) this.events.getSortings() / sum,
				(double) this.events.getDuplications() / sum,
				(double) this.events.getHostswitches() / sum };
		return result;
	}

}
